// pivCardEdge: ANSI C implementation of the API in SP 800-73, Chapter 7
//
// COPYRIGHT
// ---------
// The software was developed by the National Institute of Standards and 
// Technology (NIST), an agency of the Federal Government.  Pursuant to 
// Title 15 United States Code Section 105, works of NIST are not subject
// to copyright protection in the United States and are considered to be
// in the public domain.  As a result, a formal license is not needed to
// use the software.
// 
// The PIV Middleware Reference Implementation software is provided
// by NIST as a service and is expressly provided "AS IS."  
// NIST MAKES NO WARRANTY OF ANY KIND, EXPRESS, IMPLIED OR STATUTORY, 
// INCLUDING, WITHOUT LIMITATION, THE IMPLIED WARRANTY OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE, NON-INFRINGEMENT AND DATA ACCURACY.
// NIST does not warrant or make any representations regarding the use of
// the software or the results thereof, including but not limited to the
// correctness, accuracy, reliability or usefulness of the software.
// 
// Permission to use this software is contingent upon your acceptance of 
// the terms of this agreement and upon your providing appropriate
// acknowledgments of NIST's ownership of the software.
#ifndef __pivCardEdge_h__
#define __pivCardEdge_h__

#ifdef __cplusplus
extern "C" {
#endif

#include "pivdefs.h"

#define CACHE_SIZE		65536
#define CACHE_ID_SIZE	255

typedef struct _DataCache{
	PIV_CARDHANDLE cardHandle;
	PIV_Byte identifier[255];
	PIV_ULong32 idLen;
	PIV_Byte functionCode;
	PIV_ULong32 dataLen;
	PIV_Byte data[CACHE_SIZE];
} DataCache, *PDataCache;

typedef struct _PIV_RefData {
	PIV_Byte oldData[8];
	PIV_Byte newData[8];
} PIV_RefData, *PIV_PRefData;

typedef struct _PIV_PIN {
	PIV_Byte PIN[8];
} PIV_PIN, *PIV_PPIN;

/* Function Codes */
#define SELECT_APPLICATION                 1
#define GET_DATA                           2
#define VERIFY_PIN                         3
#define CHANGE_REFERENCE_DATA              4
#define RESET_RETRY_COUNTER                5
#define GENERAL_AUTHENTICATE               6
#define PUT_DATA                           7
#define GENERATE_ASYMMETRIC_KEY_PAIR       8

#define PIV_CMID_RSA1024        0x06
#define PIV_CMID_RSA2048        0x07
#define PIV_CMID_RSA3072        0x05
#define PIV_CMID_ECC_P_224      0x0E
#define PIV_CMID_ECC_K_233      0x0F
#define PIV_CMID_ECC_B_233      0x10
#define	PIV_CMID_ECC_P_256      0x11
#define PIV_CMID_ECC_K_283      0x12
#define PIV_CMID_ECC_P_283      0x13

#define PIV_GEN_AUTH_WITNESS               0x80
#define PIV_GEN_AUTH_CHALLENGE             0x81
#define PIV_GEN_AUTH_RESPONSE              0x82
#define PIV_GEN_AUTH_COMMITTED_CHALLENGE   0x83
#define PIV_GEN_AUTH_AUTHENTICATION_CODE   0x84



#define PIV_SW_SUCCESS					0x9000
#define PIV_SW_SUCCESS_DATA				0x6100
#define PIV_SW_VERIFY_FAILED			0x63C0
#define PIV_SW_WARNING					0x6300
#define PIV_SW_COMMUNICATION_ERROR		0x6800
#define PIV_SW_SECURITY_CONDITION_NOT_SATISFIED		0x6982
#define PIV_SW_AUTHENTICATION_METHOD_BLOCKED		0x6983
#define PIV_SW_CONDITIONS_OF_USE_NOT_SATISFIED		0x6985

#define PIV_SW_INCORRECT_PARAMETER_IN_DATA_FIELD	0x6A80
#define PIV_SW_FUNCTION_NOT_SUPPORTED				0x6A81
#define PIV_SW_DATA_OBJECT_NOT_FOUND				0x6A82
#define PIV_SW_APPLICATION_NOT_FOUND				0x6A82
#define PIV_SW_NOT_ENOUGH_MEMORY					0x6A84
#define PIV_SW_INCORRECT_PARAMETER_IN_P1_OR_P2		0x6A86
#define PIV_SW_KEY_REFERENCE_NOT_FOUND				0x6A88


StatusWord pivCardCmdSelectApplication (PIV_CARDHANDLE cardHandle, PIV_Byte *AID, PIV_Byte aidLength, PIV_Byte *applicationPropertyTemplate, PIV_ULong32 *pPropertyTemplateLength);

StatusWord pivCardCmdGetData (PIV_CARDHANDLE cardHandle, PIV_Byte *OID, PIV_Byte oidLength, PIV_Byte *data, PIV_ULong32 *pDataLength);

StatusWord pivCardCmdVerifyPin (PIV_CARDHANDLE cardHandle, PIV_Byte keyReference, PIV_PIN *pin);

StatusWord pivCardCmdChangeReferenceData (PIV_CARDHANDLE cardHandle, PIV_Byte keyReference, PIV_RefData *pRefData);

StatusWord pivCardCmdResetRetryCounter (PIV_CARDHANDLE cardHandle, PIV_Byte keyReference, PIV_Byte *data);

StatusWord pivCardCmdGeneralAuthenticate(PIV_CARDHANDLE cardHandle, PIV_Byte algorithmReference, PIV_Byte keyReference, PIV_Byte *cryptoInput, PIV_ULong32 inputLength, PIV_Byte *authData, PIV_ULong32 *pAuthDataLength); 

StatusWord pivCardCmdPutData (PIV_CARDHANDLE cardHandle, PIV_Byte *oid, PIV_ULong32 oidLen, PIV_Byte *data, PIV_ULong32 dataLength, PIV_Byte *authData, PIV_ULong32 *pAuthDataLength);

StatusWord pivCardCmdGenerateAsymmetricKeyPair(PIV_CARDHANDLE cardHandle, PIV_Byte keyReference, PIV_Byte cryptoMechanism, PIV_Byte *parameter, PIV_ULong32 parameterLength, PIV_Byte *publicKeyDataObjects, PIV_ULong32 *pPublicKeyDataObjectsLength);

void freeCache(PDataCache *ppDataCache);
void initializeCache(PDataCache *ppDataCache);
void clearCache(PDataCache pCache);
int compare (PIV_Byte* b1, PIV_Byte* b2, PIV_ULong32 len);
void populateCache (PDataCache *ppDataCache, PIV_CARDHANDLE cardHandle, PIV_Byte functionCode, PIV_Byte *identifier, PIV_ULong32 idLen, PIV_Byte* data, PIV_ULong32 dataLength);
int berGenerateLength (PIV_ULong32 len, PIV_Byte *buffer, PIV_ULong32 *pBufferLength);

PIV_RV PCSC_To_PIV (PIV_ULong32 sw);

PIV_RV PIV_Stringify_Response (PIV_ULong32 rv, PIV_UChar8 *errorBuffer, PIV_ULong32 *bufferLength);

#ifdef __cplusplus
}
#endif

#endif
