// PIVTLP224: a library used to select between a TLP socket card and PC/SC
//
// COPYRIGHT
// ---------
// The software was developed by the National Institute of Standards and 
// Technology (NIST), an agency of the Federal Government.  Pursuant to 
// Title 15 United States Code Section 105, works of NIST are not subject
// to copyright protection in the United States and are considered to be
// in the public domain.  As a result, a formal license is not needed to
// use the software.
// 
// The PIV Middleware Reference Implementation software is provided
// by NIST as a service and is expressly provided "AS IS."  
// NIST MAKES NO WARRANTY OF ANY KIND, EXPRESS, IMPLIED OR STATUTORY, 
// INCLUDING, WITHOUT LIMITATION, THE IMPLIED WARRANTY OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE, NON-INFRINGEMENT AND DATA ACCURACY.
// NIST does not warrant or make any representations regarding the use of
// the software or the results thereof, including but not limited to the
// correctness, accuracy, reliability or usefulness of the software.
// 
// Permission to use this software is contingent upon your acceptance of 
// the terms of this agreement and upon your providing appropriate
// acknowledgments of NIST's ownership of the software.

#ifndef __APPLE__
#include <winscard.h>
#else
#include <PCSC/wintypes.h>
#include <PCSC/winscard.h>
#endif

#include <stdlib.h>
#include "tlpclib.h"
#include "pivdefs.h"
#include "pivpcsc.h"
#include "pivtlp224.h"
#include <stdio.h>

//#define TLP_SERVER_URL      "piv.mobile-mind.com"
#define TLP_SERVER_URL		"localhost"
#define TLP_SERVER_PORT     9025
#define READERNAME_TLP224	"TLP224 Reader"
#define URL_MAX_SIZE		200

#define MAX_DATA_OBJECT_SIZE	65536

#define FALSE 0

#ifdef TLP224
PIV_ULong32 ulExistingConnections = 0;
PIV_Bool connectionLocked = 0;
#endif

static SCARDCONTEXT stLocalHContext = 0;
char tlpServerURL[URL_MAX_SIZE] = TLP_SERVER_URL;
int tlpServerPort=0;
char formed=FALSE;

DYNAMIC_LIB int setServerPort(int port) {
	tlpServerPort = port;
	return 0;
}

DYNAMIC_LIB int setServerURL (char *url) {
	if (strlen(url) > URL_MAX_SIZE)
		return -1;
	strcpy (tlpServerURL, url);
	return 0;
}

SCARDCONTEXT SCardGetHContext() {
  return stLocalHContext;
}

DYNAMIC_LIB PIV_RV SCardHookConnect(LPCSTR readerName, DWORD dwShareMode, DWORD dwPreferredProtocols, LPSCARDHANDLE phCard, LPDWORD pdwActiveProtocol)
{
#ifdef TLP224
	PIV_RV rv = SCARD_S_SUCCESS;
	if(ulExistingConnections > 0)
	{
		//dwShareMode is the mode for the new connection
		if( dwShareMode == SCARD_SHARE_SHARED ) 
		{
			if( connectionLocked )
				return SCARD_E_SHARING_VIOLATION;
			++ulExistingConnections;
			return rv;
		} 
		else //not shared connection and we have one, no matter of which type!
			return SCARD_E_SHARING_VIOLATION;
	}
	
	rv = TLPCLIB_SCardConnect(tlpServerURL, tlpServerPort);
	*pdwActiveProtocol = SCARD_PROTOCOL_T0;
	if (rv != 0)
		return rv;
	formed=TRUE;
	++ulExistingConnections;
	
	//if current connection is SCARD_SHARE_EXCLUSIVE, lock it.
	if( dwShareMode == SCARD_SHARE_EXCLUSIVE )
		connectionLocked = 1;

	return SCARD_S_SUCCESS;
#else
	return SCardConnect(stLocalHContext, readerName, dwShareMode,  
		                dwPreferredProtocols, 
						phCard, pdwActiveProtocol);
#endif

}

DYNAMIC_LIB PIV_RV SCardHookReconnect(SCARDHANDLE hCard, DWORD dwShareMode, DWORD dwPreferredProtocols, DWORD dwInitialization,LPDWORD pdwActiveProtocol ) {
#ifdef TLP224
    if (hCard == 0)
	{
	  if (dwInitialization == SCARD_RESET_CARD || dwInitialization == SCARD_UNPOWER_CARD)
	  {
         TLPCLIB_SCardDisconnect();
		 --ulExistingConnections;
		 if (ulExistingConnections == 0) //if no more connections, unlock. If we still have connections, they were not exclusive then
			 connectionLocked = 0;
	  }
#ifdef WIN32
	   Sleep(800);
#else
	   sleep(1);
#endif
	   TLPCLIB_SCardConnect(tlpServerURL, tlpServerPort);
	   ++ulExistingConnections;
	   if( dwShareMode == SCARD_SHARE_EXCLUSIVE )
		   connectionLocked = 1;
	   *pdwActiveProtocol = SCARD_PROTOCOL_T0;
	   return SCARD_S_SUCCESS;
    }
    else 
		return SCARD_E_INVALID_HANDLE;
#else
	return SCardReconnect ( hCard, dwShareMode, dwPreferredProtocols, 
							dwInitialization, pdwActiveProtocol);
#endif

}

DYNAMIC_LIB PIV_RV SCardHookDisconnect(SCARDHANDLE hCard)
{
#ifdef TLP224
	if (hCard == 0) {
		//if there is no connection to disconnect
		if(ulExistingConnections == 0)
			return SCARD_E_INVALID_HANDLE;
	   TLPCLIB_SCardDisconnect();
	   --ulExistingConnections;
	   // if disconecting from shared connections, they were not locked
	   // but if we have no more connections we unlock what we had before
	   // in case it was exclusive connection.
	   if(ulExistingConnections == 0)
		   connectionLocked = 0;
	   formed=FALSE;
	   return SCARD_S_SUCCESS;
	}
	else 
		return SCARD_E_INVALID_HANDLE;
#else
	return SCardDisconnect(hCard, SCARD_LEAVE_CARD);
#endif

}

DYNAMIC_LIB PIV_RV SCardHookStatus(SCARDHANDLE hCard, LPDWORD pdwState)
{
#ifdef TLP224
	if (hCard == 0) {
       *pdwState = SCARD_PRESENT;
	   return SCARD_S_SUCCESS;
	}
	else 
		return SCARD_E_INVALID_HANDLE;
#else
  char szReaderName[PIV_MAX_READERNAME];
  unsigned char pbAtr[PIV_MAX_ATR_SIZE];
  DWORD pdwProtocol;
  DWORD pcchReaderLen = PIV_MAX_READERNAME;
  DWORD pcbAtrLen = PIV_MAX_ATR_SIZE;
  return SCardStatus(hCard, szReaderName, &pcchReaderLen, pdwState,
					   &pdwProtocol, pbAtr, &pcbAtrLen);
#endif
}

DYNAMIC_LIB PIV_RV SCardHookListReaders(LPSTR mszReaders, LPDWORD pcchReaders)
{
#ifdef TLP224
	if ((mszReaders == NULL) ||
	    (*pcchReaders < strlen(READERNAME_TLP224) + 2))
	{
		*pcchReaders = (DWORD)strlen(READERNAME_TLP224) + 2;
		return SCARD_E_INSUFFICIENT_BUFFER;
	} 
	else
	{
		strcpy(mszReaders, READERNAME_TLP224);
		*pcchReaders = (DWORD)strlen(READERNAME_TLP224) + 2;
		return SCARD_S_SUCCESS;
	}
		
#else
	return SCardListReaders(stLocalHContext, NULL, mszReaders, pcchReaders);
#endif

}

DYNAMIC_LIB PIV_RV SCardHookEstablish()
{

#ifdef TLP224
	return SCARD_S_SUCCESS;
#else
	return SCardEstablishContext(SCARD_SCOPE_SYSTEM, NULL, NULL, &stLocalHContext);
#endif
}

DYNAMIC_LIB PIV_RV SCardHookRelease()
{

#ifdef TLP224
	return SCARD_S_SUCCESS;
#else
	return SCardReleaseContext(stLocalHContext);
#endif

}

DYNAMIC_LIB PIV_RV SCardHookBeginTransaction(SCARDHANDLE hCard)
{

#ifdef TLP224
	if (hCard == 0)
	   return SCARD_S_SUCCESS;
	else 
		return SCARD_E_INVALID_HANDLE;
#else
	return SCardBeginTransaction(hCard);
#endif

}

DYNAMIC_LIB PIV_RV SCardHookEndTransaction(SCARDHANDLE hCard)
{

#ifdef TLP224
	if (hCard == 0)
	   return SCARD_S_SUCCESS;
	else
		return SCARD_E_INVALID_HANDLE;
#else
	return SCardEndTransaction(hCard, SCARD_LEAVE_CARD);
#endif

}


DYNAMIC_LIB PIV_RV SCardHookExchangeAPDU( SCARDHANDLE hCard, PCSCLPTransmitBuffer transmitBuffer ) {
  long rv;
  unsigned long originalLength;
  unsigned char getResponse[5] = {0x00, 0xC0, 0x00, 0x00, 0x00};

#ifdef APDU_DEBUG
  unsigned int i;
#endif

  DWORD status = 0;
  DWORD protocol = SCARD_PROTOCOL_T0; //make it the default protocol

  CheckPCSCStatus(hCard, &status, &protocol);

  if(protocol == SCARD_PROTOCOL_T0)
	transmitBuffer->ioType = (LPSCARD_IO_REQUEST)SCARD_PCI_T0;
  else if(protocol == SCARD_PROTOCOL_T1)
	transmitBuffer->ioType = (LPSCARD_IO_REQUEST)SCARD_PCI_T1;
  else
	printf("Unknown protocol used by the program\n"); //Case if it's not T0 or T1 if that's possible?
  originalLength = transmitBuffer->apduResponseSize;

/*#ifdef TLP224  
  transmitBuffer->buffer[(transmitBuffer->bufferSize)++]=0;
  transmitBuffer->buffer[(transmitBuffer->bufferSize)++]=0;
#endif*/

#ifdef APDU_DEBUG
	printf("[%03d]->: ", transmitBuffer->bufferSize);
    for (i=0; i < transmitBuffer->bufferSize; i++) {
		printf("%02X ", transmitBuffer->buffer[i]);
    }
	printf("\n");
#endif

#ifdef TLP224
	if (hCard == 0) {
		  TLPCLIB_SCardTransmit(transmitBuffer->buffer, transmitBuffer->bufferSize,
			  transmitBuffer->apduResponse, &transmitBuffer->apduResponseSize);
		rv = SCARD_S_SUCCESS;
	}
	else 
		rv = SCARD_E_INVALID_HANDLE;
#else
	transmitBuffer->apduResponse[0]=0;transmitBuffer->apduResponse[1]=0;
	rv = SCardTransmit(hCard, 
	      transmitBuffer->ioType,
		  transmitBuffer->buffer, 
		  transmitBuffer->bufferSize,
		  0,
		  transmitBuffer->apduResponse, 
		  &(transmitBuffer->apduResponseSize) );
#endif

#ifdef APDU_DEBUG
	if (transmitBuffer->apduResponse[0]!=0) {
		printf("[%03d]<-: ", transmitBuffer->apduResponseSize);
		for (i=0; i < transmitBuffer->apduResponseSize; i++) {
			printf("%02X ", transmitBuffer->apduResponse[i]);
		}
		printf("\n");
	}
#endif
	return rv;
}


PIV_RV CheckPCSCStatus(SCARDHANDLE hCard, PDWORD pcscStatus, PDWORD pcscProtocol) {

  PIV_RV rv;
  char statusReaderName[PIV_MAX_READERNAME];
  DWORD statusReaderNameLen;
  unsigned char statusAtr[PIV_MAX_ATR_SIZE];
  DWORD statusAtrLen;
  
  statusAtrLen        = sizeof(statusAtr);
  statusReaderNameLen = sizeof(statusReaderName);
  
  rv = SCardStatus(hCard, statusReaderName, &statusReaderNameLen,
                   pcscStatus, pcscProtocol, statusAtr,
                   &statusAtrLen);
                                        
  return rv;
}

DYNAMIC_LIB char* Stringify_Error (PIV_RV errorCode) {
	
	switch (errorCode) {
#ifdef WIN32
		case ERROR_BROKEN_PIPE: 	return "ERROR_BROKEN_PIPE: The client attempted a smart card operation in a remote session, such as a client session running on a terminal server, and the operating system in use does not support smart card redirection.";
		case SCARD_E_BAD_SEEK:		return "SCARD_E_BAD_SEEK: An error occurred in setting the smart card file object pointer.";
#endif
		case SCARD_E_CANCELLED: 	return "SCARD_E_CANCELLED: The action was canceled by an SCardCancel request.";
		case SCARD_E_CANT_DISPOSE:	return "SCARD_E_CANT_DISPOSE: 	The system could not dispose of the media in the requested manner.";
		case SCARD_E_CARD_UNSUPPORTED:	return "SCARD_E_CARD_UNSUPPORTED: The smart card does not meet minimal requirements for support.";
#ifdef WIN32
		case SCARD_E_CERTIFICATE_UNAVAILABLE: 	return "SCARD_E_CERTIFICATE_UNAVAILABLE: The requested certificate could not be obtained.";
		case SCARD_E_COMM_DATA_LOST: 	return "SCARD_E_COMM_DATA_LOST: A communications error with the smart card has been detected.";
		case SCARD_E_DIR_NOT_FOUND:		return "SCARD_E_DIR_NOT_FOUND: The specified directory does not exist in the smart card.";
#endif
		case SCARD_E_DUPLICATE_READER: 	return "SCARD_E_DUPLICATE_READER: The reader driver did not produce a unique reader name.";
#ifdef WIN32
		case SCARD_E_FILE_NOT_FOUND: 	return "SCARD_E_FILE_NOT_FOUND: The specified file does not exist in the smart card.";
		case SCARD_E_ICC_CREATEORDER: 	return "SCARD_E_ICC_CREATEORDER: The requested order of object creation is not supported.";
		case SCARD_E_ICC_INSTALLATION: 	return "SCARD_E_ICC_INSTALLATION: No primary provider can be found for the smart card.";
#endif
		case SCARD_E_INSUFFICIENT_BUFFER: 	return "SCARD_E_INSUFFICIENT_BUFFER: The data buffer for returned data is too small for the returned data.";
		case SCARD_E_INVALID_ATR: 	return "SCARD_E_INVALID_ATR: An ATR string obtained from the registry is not a valid ATR string.";
#ifdef WIN32
		case SCARD_E_INVALID_CHV: 	return "SCARD_E_INVALID_CHV: The supplied PIN is incorrect.";
#endif
		case SCARD_E_INVALID_HANDLE: 	return "SCARD_E_INVALID_HANDLE: The supplied handle was invalid.";
		case SCARD_E_INVALID_PARAMETER: return "SCARD_E_INVALID_PARAMETER: One or more of the supplied parameters could not be properly interpreted.";
		case SCARD_E_INVALID_TARGET: 	return "SCARD_E_INVALID_TARGET: Registry startup information is missing or invalid.";
		case SCARD_E_INVALID_VALUE: 	return "SCARD_E_INVALID_VALUE: One or more of the supplied parameter values could not be properly interpreted.";
#ifdef WIN32
		case SCARD_E_NO_ACCESS: 	return "SCARD_E_NO_ACCESS: Access is denied to the file.";
		case SCARD_E_NO_DIR: 	return "SCARD_E_NO_DIR: The supplied path does not represent a smart card directory.";
		case SCARD_E_NO_FILE: 	return "SCARD_E_NO_FILE: The supplied path does not represent a smart card file.";
#endif
		case SCARD_E_NO_MEMORY: return "SCARD_E_NO_MEMORY: Not enough memory available to complete this command.";
#ifdef WIN32
		case SCARD_E_NO_READERS_AVAILABLE: 	return "SCARD_E_NO_READERS_AVAILABLE: No smart card reader is available.";
#endif
		case SCARD_E_NO_SERVICE: 	return "SCARD_E_NO_SERVICE: The smart card resource manager is not running.";
		case SCARD_E_NO_SMARTCARD: 	return "SCARD_E_NO_SMARTCARD: The operation requires a smart card, but no smart card is currently in the device.";
#ifdef WIN32
		case SCARD_E_NO_SUCH_CERTIFICATE: 	return "SCARD_E_NO_SUCH_CERTIFICATE: The requested certificate does not exist.";
#endif
		case SCARD_E_NOT_READY: 	return "SCARD_E_NOT_READY: The reader or card is not ready to accept commands.";
		case SCARD_E_NOT_TRANSACTED: 	return "SCARD_E_NOT_TRANSACTED: An attempt was made to end a nonexistent transaction.";
		case SCARD_E_PCI_TOO_SMALL: 	return "SCARD_E_PCI_TOO_SMALL: The PCI receive buffer was too small.";
		case SCARD_E_PROTO_MISMATCH: 	return "SCARD_E_PROTO_MISMATCH: The requested protocols are incompatible with the protocol currently in use with the card.";
		case SCARD_E_READER_UNAVAILABLE: 	return "SCARD_E_READER_UNAVAILABLE: The specified reader is not currently available for use.";
		case SCARD_E_READER_UNSUPPORTED: 	return "SCARD_E_READER_UNSUPPORTED: The reader driver does not meet minimal requirements for support.";
		case SCARD_E_SERVICE_STOPPED:	return "SCARD_D_SERVICE_STOPPED: The smart card resource manager has shut down.";
		case SCARD_E_SHARING_VIOLATION: return "SCARD_E_SHARING_VIOLATION: The smart card cannot be accessed because of other outstanding connections.";
		case SCARD_E_SYSTEM_CANCELLED: 	return "SCARD_E_SYSTEM_CANCELLED: The action was canceled by the system, presumably to log off or shut down.";
		case SCARD_E_TIMEOUT: 	return "SCARD_E_TIMEOUT: The user-specified time-out value has expired.";
#ifdef WIN32
		case SCARD_E_UNEXPECTED: 	return "SCARD_E_UNEXPECTED: An unexpected card error has occurred.";
#endif
		case SCARD_E_UNKNOWN_CARD: 	return "SCARD_E_UNKNOWN_CARD: The specified smart card name is not recognized.";
		case SCARD_E_UNKNOWN_READER: 	return "SCARD_E_UNKNOWN_READER: The specified reader name is not recognized.";
#ifdef WIN32
		case SCARD_E_UNKNOWN_RES_MNG: 	return "SCARD_E_UNKNOWN_RES_MNG: An unrecognized error code was returned from a layered component.";
		case SCARD_E_UNSUPPORTED_FEATURE: 	return "SCARD_E_UNSUPPORTED_FEATIRE: This smart card does not support the requested feature.";
		case SCARD_E_WRITE_TOO_MANY: 	return "SCARD_E_WRITE_TOO_MANY: An attempt was made to write more data than would fit in the target object.";
#endif
		case SCARD_F_COMM_ERROR: 	return "SCARD_F_COMM_ERROR: An internal communications error has been detected.";
		case SCARD_F_INTERNAL_ERROR: 	return "SCARD_F_INTERNAL_ERROR: An internal consistency check failed.";
		case SCARD_F_UNKNOWN_ERROR: 	return "SCARD_F_UNKNOWN_ERROR: An internal error has been detected, but the source is unknown.";
		case SCARD_F_WAITED_TOO_LONG: 	return "SCARD_F_WAITED_TOO_LONG: An internal consistency timer has expired.";
#ifdef WIN32
		case SCARD_P_SHUTDOWN: 	return "SCARD_P_SHUTDOWN: The operation has been aborted to allow the server application to exit.";
		case SCARD_S_SUCCESS: 	return "SCARD_S_SUCCESS: No error was encountered.";
		case SCARD_W_CANCELLED_BY_USER: return "SCARD_W_CANCELLED_BY_USER: The action was canceled by the user.";
		case SCARD_W_CHV_BLOCKED: 	return "SCARD_W_CHV_BLOCKED: The card cannot be accessed because the maximum number of PIN entry attempts has been reached.";
		case SCARD_W_EOF: 	return "SCARD_W_EOF: The end of the smart card file has been reached.";
#endif
		case SCARD_W_REMOVED_CARD: 	return "SCARD_W_REMOVED_CARD: The smart card has been removed, so further communication is not possible.";
#ifdef WIN32
		case SCARD_W_RESET_CARD: 	return "SCARD_W_RESET_CARD: The smart card has been reset, so any shared state information is invalid.";
		case SCARD_W_SECURITY_VIOLATION: 	return "SCARD_W_SECURITY_VIOLATION: Access was denied because of a security violation.";
		case SCARD_W_UNPOWERED_CARD: 	return "SCARD_W_UNPOWERED_CARD: Power has been removed from the smart card, so that further communication is not possible.";
		case SCARD_W_UNRESPONSIVE_CARD: 	return "SCARD_W_UNRESPONSIVE_CARD: The smart card is not responding to a reset.";
		case SCARD_W_UNSUPPORTED_CARD: 	return "SCARD_W_UNSUPPORTED_CARD: The reader cannot communicate with the card, due to ATR string configuration conflicts.";
		case SCARD_W_WRONG_CHV: 	return "SCARD_W_WRONG_CHV: The card cannot be accessed because the wrong PIN was presented.";
#endif
		default: return "unknown error...";
	}
	return "";
}
