// APDU_Packet - Data Structures for Network-Word-Order APDU Wrapper Packets
//
// COPYRIGHT
// ---------
// The software was developed by the National Institute of Standards and 
// Technology (NIST), an agency of the Federal Government.  Pursuant to 
// Title 15 United States Code Section 105, works of NIST are not subject
// to copyright protection in the United States and are considered to be
// in the public domain.  As a result, a formal license is not needed to
// use the software.
// 
// The Government Smart Card Reference Implementation software is provided
// by NIST as a service and is expressly provided "AS IS."  
// NIST MAKES NO WARRANTY OF ANY KIND, EXPRESS, IMPLIED OR STATUTORY, 
// INCLUDING, WITHOUT LIMITATION, THE IMPLIED WARRANTY OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE, NON-INFRINGEMENT AND DATA ACCURACY.
// NIST does not warrant or make any representations regarding the use of
// the software or the results thereof, including but not limited to the
// correctness, accuracy, reliability or usefulness of the software.
// 
// Permission to use this software is contingent upon your acceptance of 
// the terms of this agreement and upon your providing appropriate
// acknowledgments of NIST's ownership of the software.

#include <APDU_Packet.h>
#include <strstream>

string APDU_Packet::spell ( void ) const
{
  ostrstream spelling_stream;

  spelling_stream.width(3);
  spelling_stream << static_cast < unsigned long > ( get_APDU_length ( ) ) ;

  spelling_stream.unsetf( ios_base::dec );
  spelling_stream.setf( ios_base::hex );
  spelling_stream.fill('0');

  if ( 0 < get_APDU_length ( ) )
    {
        spelling_stream << ":" ;
        for ( size_t i = 0 ; i < get_APDU_length ( ) ; i ++ )
          {
            spelling_stream << " 0x" ;
            spelling_stream.width(2);
            spelling_stream << static_cast < short > ( get_APDU_octet ( i ) ) ;
          }
    }

  spelling_stream << ends ;

  return spelling_stream . str ( ) ;
}


void APDU_Packet::log (const char * prefix) const
{
  log_message ( "%s%s\n" , prefix , spell ( ) . c_str ( ) );
}

void APDU_Packet::debug (const char * prefix) const
{
  debug_message ( "%s%s\n" , prefix , spell ( ) . c_str ( ) );
}

void APDU_Packet::error (const char * prefix) const
{
  error_message ( "%s%s\n" , prefix , spell ( ) . c_str ( ) );
}

