// Card Registry
//
// COPYRIGHT
// ---------
// The software was developed by the National Institute of Standards and 
// Technology (NIST), an agency of the Federal Government.  Pursuant to 
// Title 15 United States Code Section 105, works of NIST are not subject
// to copyright protection in the United States and are considered to be
// in the public domain.  As a result, a formal license is not needed to
// use the software.
// 
// The Government Smart Card Reference Implementation software is provided
// by NIST as a service and is expressly provided "AS IS."  
// NIST MAKES NO WARRANTY OF ANY KIND, EXPRESS, IMPLIED OR STATUTORY, 
// INCLUDING, WITHOUT LIMITATION, THE IMPLIED WARRANTY OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE, NON-INFRINGEMENT AND DATA ACCURACY.
// NIST does not warrant or make any representations regarding the use of
// the software or the results thereof, including but not limited to the
// correctness, accuracy, reliability or usefulness of the software.
// 
// Permission to use this software is contingent upon your acceptance of 
// the terms of this agreement and upon your providing appropriate
// acknowledgments of NIST's ownership of the software.

#include "Card_Registry.h"


bool
Card_Registry::add_entry ( const Octet_String & AID ,
                           Registry_Entry::Content_Type content_type )
{
  // make sure entry does not already exist
  try
  {
    find_entry ( AID, content_type ) ;
    
    return false ; // fail if entry already exists
  }
  catch ( Registry_Entry_Not_Found_Exception )
  {
    Registry_Entry_Pointer new_entry_pointer =
      Registry_Entry_Pointer::object_pointer_to_indexed_pointer
        ( new Registry_Entry ( AID, content_type ) ) ;

    the_entry_pointers . push_back ( new_entry_pointer ) ;
  
    return true;
  }
  
}


bool
Card_Registry::delete_entry ( const Octet_String & AID ,
                              Registry_Entry::Content_Type content_type )
{
  // TODO: implement delete;
  return false;
}


Registry_Entry::Manager
Card_Registry::get_manager ( const Octet_String & AID ,
                             Registry_Entry::Content_Type content_type )
{
  try
  {
    Registry_Entry & entry = find_entry ( AID, content_type ) ;
    
    return entry . manager ;
  }
  catch ( Registry_Entry_Not_Found_Exception )
  {
    return Registry_Entry::manager_invalid;
  }
}

Registry_Entry::Manager
Card_Registry::default_manager ( )
{
  return Registry_Entry::manager_javacard;
}

bool
Card_Registry::set_manager ( const Octet_String & AID ,
                             Registry_Entry::Content_Type content_type,
                             Registry_Entry::Manager manager )
{
  try
  {
    Registry_Entry & entry = find_entry ( AID, content_type ) ;

    entry . manager = manager ;

    return true ;
  }
  catch ( Registry_Entry_Not_Found_Exception )
  {
    return false ;
  }
  
}


Registry_Entry::Lifecycle
Card_Registry::get_lifecycle ( const Octet_String & AID ,
                               Registry_Entry::Content_Type content_type )
{
  return Registry_Entry::lifecycle_invalid;
}


bool
Card_Registry::set_lifecycle ( const Octet_String & AID ,
                               Registry_Entry::Content_Type content_type,
                               Registry_Entry::Lifecycle lifecycle )
{
  return false;
}


unsigned short
Card_Registry::get_privileges ( const Octet_String & AID ,
                                Registry_Entry::Content_Type content_type )
{
  return 0;
}


bool
Card_Registry::set_privileges ( const Octet_String & AID ,
                                Registry_Entry::Content_Type content_type,
                                unsigned short privileges )
{
  return false;
}


unsigned int
Card_Registry::get_resources   ( const Octet_String & AID ,
                                 Registry_Entry::Content_Type content_type )
{
  return 0;
}


bool
Card_Registry::set_resources   ( const Octet_String & AID ,
                                 Registry_Entry::Content_Type content_type,
                                 unsigned int resources )
{
  return false ;
}


bool
Card_Registry::store_data_block ( const Octet_String & AID  ,
                                  Registry_Entry::Content_Type content_type ,
                                  const Octet_String & data_block )
{
  try
  {
    // TODO: enforce only content type load file
    Registry_Entry & entry = find_entry ( AID, content_type ) ;
    
    entry . data_blocks . push_back ( data_block ) ;

    return true ;
  }
  catch ( Registry_Entry_Not_Found_Exception )
  {
    return false ; 
  }  
}



Load_Module *
Card_Registry::get_data_blocks ( const Octet_String & AID  ,
                                 Registry_Entry::Content_Type content_type )
{
  try
  {
    // TODO: enforce only content type load file
    Registry_Entry & entry = find_entry ( AID, content_type ) ;
    
    return & entry . data_blocks ;
  }
  catch ( Registry_Entry_Not_Found_Exception )
  {
    return 0 ;
  }
}


Registry_Entry &
Card_Registry::find_entry ( const Octet_String & AID  ,
                            Registry_Entry::Content_Type content_type )
{
  for ( Registry_Entry_Pointer_List :: iterator p = the_entry_pointers . begin();
        p != the_entry_pointers . end( ) ;
        p++ )

    if ( ( ( * p ) -> content_type == content_type )
         && ( ( * p ) -> AID == AID ) )
      return * * p ;

  throw Registry_Entry_Not_Found_Exception ( ) ;
}
