// File System
//
// COPYRIGHT
// ---------
// The software was developed by the National Institute of Standards and 
// Technology (NIST), an agency of the Federal Government.  Pursuant to 
// Title 15 United States Code Section 105, works of NIST are not subject
// to copyright protection in the United States and are considered to be
// in the public domain.  As a result, a formal license is not needed to
// use the software.
// 
// The Government Smart Card Reference Implementation software is provided
// by NIST as a service and is expressly provided "AS IS."  
// NIST MAKES NO WARRANTY OF ANY KIND, EXPRESS, IMPLIED OR STATUTORY, 
// INCLUDING, WITHOUT LIMITATION, THE IMPLIED WARRANTY OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE, NON-INFRINGEMENT AND DATA ACCURACY.
// NIST does not warrant or make any representations regarding the use of
// the software or the results thereof, including but not limited to the
// correctness, accuracy, reliability or usefulness of the software.
// 
// Permission to use this software is contingent upon your acceptance of 
// the terms of this agreement and upon your providing appropriate
// acknowledgments of NIST's ownership of the software.

#include <File_System.h>

void Elementary_File::dump ( File_System * the_file_system_pointer , unsigned int indentation )
  {
    debug_message ( "%*s/%04X%s\n" ,
                    File::dump_indent_spaces * indentation , "" , 
                    get_FCP ( ) . file_identifier ( ) ,
                    the_file_system_pointer -> is_current_file ( * this ) 
                    ? "  <<=== CURRENT FILE ===" : "" ) ;

    string data_indentation ( File::dump_indent_spaces * ( indentation + 1 ) , ' ' );
    debug_message ( "%s\n" ,
                    Octet_String ( the_data ) 
                      . spell_with_indentation ( data_indentation ) 
                         . c_str ( ) ) ;
  } ;

void Data_Object::dump ( File_System * the_file_system_pointer , unsigned int indentation )
  {
    debug_message ( "%*s/%06X\n" ,
                    File::dump_indent_spaces * indentation , "" , 
                    get_FCP ( ) . data_object_tag ( ) ) ;

    string data_indentation ( File::dump_indent_spaces * ( indentation + 1 ) , ' ' );
    debug_message ( "%s\n" ,
                    Octet_String ( the_data ) 
                      . spell_with_indentation ( data_indentation ) 
                         . c_str ( ) ) ;
  } ;

void Dedicated_File::dump ( File_System * the_file_system_pointer , unsigned int indentation )
  {
    debug_message ( "%*s/%04X%s\n" ,
                    File::dump_indent_spaces * indentation, "",  
                    get_FCP ( ) . file_identifier ( ) ,
                    the_file_system_pointer -> is_current_file ( * this ) 
                    ? "  <<=== CURRENT FILE ===" : "" ) ;

    for ( Dedicated_File_Pointers::iterator pd = the_dedicated_file_pointers . begin ( ) ;
          pd != the_dedicated_file_pointers . end ( ) ;
          pd ++ )
      ( * pd ) -> dump ( the_file_system_pointer , indentation + 1 ) ;
    
    for ( Elementary_File_Pointers::iterator pd = the_elementary_file_pointers . begin ( ) ;
          pd != the_elementary_file_pointers . end ( ) ;
          pd ++ )
      ( * pd ) -> dump ( the_file_system_pointer , indentation + 1 ) ;

    for ( Data_Object_Pointers::iterator pd = the_data_object_pointers . begin ( ) ;
          pd != the_data_object_pointers . end ( ) ;
          pd ++ )
      ( * pd ) -> dump ( the_file_system_pointer , indentation + 1 ) ;
  } ;


void File_System::dump ( )
  {
    debug_message ( "File System:\n"
                    "/-------------------------------------------------\n");
    the_master_file_pointer -> dump ( this , 0 ) ;
    debug_message ( "\\-------------------------------------------------\n\n");
    if ( Security_System_Pointer ( ) != the_security_system_pointer )
      the_security_system_pointer -> dump ( ) ;
  } ;


void File_System::verify_authorization ( const Security_Attribute_Pointer & security_attribute_pointer , 
                                          Octet access_mode )
  {
    if ( Security_Attribute_Pointer ( ) 
           == security_attribute_pointer )
      throw RAPDU_Packet::SW_SECURITY_STATUS_NOT_SATISFIED ;

    if ( ! ( the_security_system_pointer
               -> authorized ( security_attribute_pointer , 
                               access_mode ) ) )
      throw RAPDU_Packet::SW_SECURITY_STATUS_NOT_SATISFIED ;
  } ;

void File_System::verify_data_object_authorization ( const BER_TLV_Tag & tag,
                                                     Octet access_mode )
  {
    set_current_data_object_pointer ( tag ) ;

    const Security_Attribute_Pointer the_security_attribute_pointer =
      the_current_data_object_pointer
          -> get_FCP ( ) . security_attribute_pointer ( );
    verify_authorization ( the_security_attribute_pointer , access_mode ) ;
  } ;

void File_System::verify_data_object_authorization ( Octet access_mode )
  {
    const Security_Attribute_Pointer the_security_attribute_pointer =
      the_current_data_object_pointer 
          -> get_FCP ( ) . security_attribute_pointer ( );
    verify_authorization ( the_security_attribute_pointer , access_mode ) ;
  } ;

void File_System::verify_elementary_file_authorization ( Octet access_mode )
  {
    const Security_Attribute_Pointer the_security_attribute_pointer =
      the_current_elementary_file_pointer 
          -> get_FCP ( ) . security_attribute_pointer ( );
    verify_authorization ( the_security_attribute_pointer , access_mode ) ;
  } ;

void File_System::verify_dedicated_file_authorization ( Octet access_mode )
  {
    const Security_Attribute_Pointer the_security_attribute_pointer =
      the_current_dedicated_file_pointer 
          -> get_FCP ( ) . security_attribute_pointer ( );
    verify_authorization ( the_security_attribute_pointer , access_mode ) ;
  } ;
