// Card Registry
//
// COPYRIGHT
// ---------
// The software was developed by the National Institute of Standards and 
// Technology (NIST), an agency of the Federal Government.  Pursuant to 
// Title 15 United States Code Section 105, works of NIST are not subject
// to copyright protection in the United States and are considered to be
// in the public domain.  As a result, a formal license is not needed to
// use the software.
// 
// The Government Smart Card Reference Implementation software is provided
// by NIST as a service and is expressly provided "AS IS."  
// NIST MAKES NO WARRANTY OF ANY KIND, EXPRESS, IMPLIED OR STATUTORY, 
// INCLUDING, WITHOUT LIMITATION, THE IMPLIED WARRANTY OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE, NON-INFRINGEMENT AND DATA ACCURACY.
// NIST does not warrant or make any representations regarding the use of
// the software or the results thereof, including but not limited to the
// correctness, accuracy, reliability or usefulness of the software.
// 
// Permission to use this software is contingent upon your acceptance of 
// the terms of this agreement and upon your providing appropriate
// acknowledgments of NIST's ownership of the software.

#if !defined(__CARD_REGISTRY_H_INCLUDED__)
#define __CARD_REGISTRY_H_INCLUDED__


#include <Manager.h>


#define MAX_AID_LENGTH  64 // TODO: use real value

class Registry_Entry_Not_Found_Exception
{
public:
  bool exp;
};


class Registry_Entry : public Object
{
public:
  enum Content_Type {
    content_type_application, // Application, Security Domain, Issuer Security Domain
    content_type_executable_load_file,
    content_type_executable_module,
    content_type_invalid
  };

  enum Manager {
    manager_javacard, // this entry is associated with the JavaCard Manager
    manager_file,     // this entry is associated with the File Manager
    manager_invalid
  };

  enum Lifecycle {
    card_ready,       // Card
    card_initialized, // Card
    card_secured,     // Card
    card_locked,      // Card
    card_terminated,  // Card
    file_loaded,   // File
    module_loaded, // Module
    application_installed,  // Application
    application_selectable, // Application
    application_locked,     // Application
    lifecycle_invalid
  };
  
  Registry_Entry ( const Octet_String & new_AID,
                   Registry_Entry::Content_Type new_content_type )
    : AID ( new_AID . length ( ) ) 
    {
      Octet_String::cpy ( AID, new_AID ) ;
      content_type = new_content_type ;
    }
  
  Content_Type content_type ;        // Application, Load File, Executable Module
  Octet_String AID ;                 // AID of this Object
  // Octet_String SD_AID ;           // AID of the associated Security Domain
  Manager manager ;                  // associated manager
  Lifecycle lifecycle ;              // life cycle status
  unsigned short privileges ;        // application privileges
  unsigned long  resources ;         // resource allocation
  Load_Module data_blocks ;          // storage for load file data blocks
  Octet load_file_format_indicator ; // 
};

typedef indexed_pointer < Registry_Entry > Registry_Entry_Pointer ;


class Card_Registry : public Object
{
public:

  bool
  add_entry ( const Octet_String & AID ,
              Registry_Entry::Content_Type content_type ) ;
    
  bool
  delete_entry ( const Octet_String & AID ,
                 Registry_Entry::Content_Type content_type ) ;

/* TODO: Implement search by AID
    Octet_String & find_entry      ( const Octet_String & AID ,
                                     unsigned short content_type );
    Octet_String & find_next_entry ( const Octet_String & AID ,
                                     unsigned short content_type );
*/

  // returns the Application Manager associated with the content.
  // returns Registry_Entry::manager_invalid if content not found.
  Registry_Entry::Manager
  get_manager ( const Octet_String & AID ,
                 Registry_Entry::Content_Type content_type );

  Registry_Entry::Manager
  default_manager (  );

  bool
  set_manager ( const Octet_String & AID  ,            // only set at add time?
                 Registry_Entry::Content_Type content_type ,
                 Registry_Entry::Manager manager );

  Registry_Entry::Lifecycle
  get_lifecycle ( const Octet_String & AID  ,
                  Registry_Entry::Content_Type content_type ) ;

  bool
  set_lifecycle ( const Octet_String & AID  ,
                  Registry_Entry::Content_Type content_type ,
                  Registry_Entry::Lifecycle lifecycle ) ;

  unsigned short
  get_privileges ( const Octet_String & AID  ,
                   Registry_Entry::Content_Type content_type ) ;

  bool
  set_privileges ( const Octet_String & AID  ,
                   Registry_Entry::Content_Type content_type ,
                   unsigned short privileges ) ;

  unsigned int
  get_resources ( const Octet_String & AID  ,
                  Registry_Entry::Content_Type content_type ) ;

  bool
  set_resources ( const Octet_String & AID  ,
                  Registry_Entry::Content_Type content_type ,
                  unsigned int resources ) ;

  bool
  store_data_block ( const Octet_String & AID  ,
                     Registry_Entry::Content_Type content_type ,
                     const Octet_String & data_block ) ;

  Load_Module *
  get_data_blocks ( const Octet_String & AID  ,
                    Registry_Entry::Content_Type content_type ) ;  

private:
  Registry_Entry &
  find_entry ( const Octet_String & AID  ,
               Registry_Entry::Content_Type content_type ) 
    throw ( Registry_Entry_Not_Found_Exception ) ;

  typedef list < Registry_Entry_Pointer , persistent_allocator < Registry_Entry_Pointer > >
    Registry_Entry_Pointer_List ;

  Registry_Entry_Pointer_List the_entry_pointers ;
  
} ;

typedef indexed_pointer < Card_Registry > Card_Registry_Pointer  ;

#endif // __CARD_REGISTRY_H_INCLUDED__

