// TLV
//
// COPYRIGHT
// ---------
// The software was developed by the National Institute of Standards and 
// Technology (NIST), an agency of the Federal Government.  Pursuant to 
// Title 15 United States Code Section 105, works of NIST are not subject
// to copyright protection in the United States and are considered to be
// in the public domain.  As a result, a formal license is not needed to
// use the software.
// 
// The Government Smart Card Reference Implementation software is provided
// by NIST as a service and is expressly provided "AS IS."  
// NIST MAKES NO WARRANTY OF ANY KIND, EXPRESS, IMPLIED OR STATUTORY, 
// INCLUDING, WITHOUT LIMITATION, THE IMPLIED WARRANTY OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE, NON-INFRINGEMENT AND DATA ACCURACY.
// NIST does not warrant or make any representations regarding the use of
// the software or the results thereof, including but not limited to the
// correctness, accuracy, reliability or usefulness of the software.
// 
// Permission to use this software is contingent upon your acceptance of 
// the terms of this agreement and upon your providing appropriate
// acknowledgments of NIST's ownership of the software.

#if !defined(__TLV_H_INCLUDED__)
#define __TLV_H_INCLUDED__

#if defined(_MSC_VER )
  // It's OK that the compiler can't automatically generate default constructors or
  // assignment operators for classes with const data members
  // We're shooting for immutability here
  #pragma warning ( disable : 4702 )  // unreachable code in <list>
  #if defined(TLV_EXPORTS)
    #define TLV_DLL _declspec(dllexport)
  #else //  defined ( TLV_EXPORTS )
    #define TLV_DLL _declspec(dllimport)
  #endif  // defined ( TLV_EXPORTS )
#else
   #define TLV_DLL
#endif

#include <list>
#include <iostream>
using namespace std;

#include <tags.h>

#include <Octet_String.h>

class TLV_DLL Malformed_TLV_Exception
{
public:
   Malformed_TLV_Exception ( const char * message, Octet_String octet_string, Memory::offset_t given_start, Memory::offset_t given_stop, Memory::offset_t actual_stop)
      : the_message ( message),
        the_octet_string ( octet_string), 
        the_given_start ( given_start), 
        the_given_stop ( given_stop), 
        the_actual_stop ( actual_stop)
   {};
   const char * the_message ;
   const Octet_String the_octet_string ;
   const Memory::offset_t the_given_start ;
   const Memory::offset_t the_given_stop ;
   const Memory::offset_t the_actual_stop ;
} ;

class TLV_DLL TLV : public Octet_String
{
public:
   TLV ( const Octet_String & octet_string , Memory::offset_t start )
      : Octet_String ( octet_string , start )
   {};
} ;

/* ISO/IEC 8825-1 multi-byte Octet_String */
class TLV_DLL BER_TLV_Tag : public Octet_String
{
public:
//
// ISO/IEC 8825-1 BER-TLV Tags
//
// 000x xxxx - Universal, primitive                     0x0n, 0x1n
// 001x xxxx - Universal, constructed                   0x2n, 0x3n
// 010x xxxx - Application, primitive                   0x4n, 0x5n
// 011x xxxx - Application, constructed                 0x6n, 0x7n
// 100x xxxx - Context-specific, primitive              0x8n, 0x9n
// 101x xxxx - Context-specific, constructed            0xAn, 0xBn
// 110x xxxx - Private, primitive                       0xCn, 0xDn
// 111x xxxx - Private, constructed                     0xEn, 0xFn
//
//

  static const int BER_TLV_Tag::category     = 0xC0 ;
  static const int BER_TLV_Tag::constructed  = 0x20 ;
  static const int BER_TLV_Tag::index        = 0x1F ;

  // Build a one-byte BER_TLV Tag
  BER_TLV_Tag ( const Octet single_octet_tag)
    // TODO: This should test that the octet is a valid single-octet tag
    // TODO: If not, it should make up a multi-byte tag
    : Octet_String ( 1 )
  {
    put_octet ( 0, single_octet_tag);
  };

  BER_TLV_Tag ( const Octet_String & octet_string , Memory::offset_t start )
    : Octet_String ( octet_string , start ,
                       index != ( octet_string [ start - octet_string . start ( ) ] & index )
                          ? start + 1
                       :  0x80 == ( octet_string [ start - octet_string . start ( ) + 1 ] & 0x80 ) 
                          ? start + 3 
                          : start + 2 )
  // TODO: more checking on tag well-formed-ness
  { } ;

  BER_TLV_Tag ( const Octet_String & octet_string )
    : Octet_String ( octet_string , octet_string . start ( ) ,
                       index != ( octet_string [ 0 ] & index )
                          ? octet_string . start ( ) + 1
                       :  0x80 == ( octet_string [ 1 ] & 0x80 ) 
                          ? octet_string . start ( ) + 3 
                          : octet_string . start ( ) + 2 )
  // TODO: more checking on tag well-formed-ness
  { } ;

   bool operator == ( const BER_TLV_Tag & other) const
   {
      return ( Octet_String) ( *this)== ( Octet_String)other;
   }

   bool operator != ( const BER_TLV_Tag & other) const
   {
      return ! ( ( * this ) == other );
   }

   bool denotes_constructed_TLV ( void) const
   {
     return 0 != ( (  the_storage -> get ( the_start ) ) & constructed  ) ;
   }

  static BER_TLV_Tag 
  generate ( Octet_String & where , const BER_TLV_Tag & tag )
    {
      Octet_String the_generated_tag ( where , where . stop ( ) - ( tag . the_stop - tag . the_start ) ) ;
      Octet_String::cpy ( the_generated_tag , tag ) ;
      where = Octet_String ( where , where . start ( ) , the_generated_tag . start ( ) ) ;
      return the_generated_tag ;
    } ;

  static BER_TLV_Tag 
  generate ( Octet_String & where , const Octet single_octet_tag )
    // TODO: This should test that the octet is a valid single-octet tag
    // TODO: If not, it should make up a multi-byte tag
    {
      return generate ( where , BER_TLV_Tag ( single_octet_tag ) ) ;
    } ;

} ;




class TLV_DLL BER_TLV_Length : public Octet_String
{
public:
   BER_TLV_Length ( const Octet_String & octet_string, offset_t start)
      : Octet_String ( octet_string , start ,
                        0x80 == ( 0x80 & octet_string [ start - octet_string . start ( )  ] )
                           ? start + 1 + ( 0x7F & octet_string [ start  - octet_string . start ( ) ] )
                           : start + 1 )
   { } ;

   BER_TLV_Length ( const Octet_String & octet_string )
      : Octet_String ( octet_string , octet_string . start ( ) ,
                        0x80 == ( 0x80 & octet_string [ 0 ] )
                           ? octet_string . start ( ) + 1 + ( 0x7F & octet_string [ 0 ] )
                           : octet_string . start ( ) + 1 )
   { } ;

   BER_TLV_Length ( size_t length ) 
     : Octet_String ( length <= 0x7F 
                        ? 1 
                      : length <= 0xFF 
                        ? 2 
                      : length <= 0xFFFF 
                        ? 3 
                      : length <= 0xFFFFFF 
                        ? 4 
                        : 5 )
     {
       if ( length <= 0x7F )
         {
           put_octet ( 0 , static_cast <Octet> ( length ) ) ;
         }
       else if ( length <= 0xFF )
         {
           put_octet ( 0 , 0x81 ) ;
           put_octet ( 1 , static_cast <Octet> ( length ) ) ;
         }
       else if ( length <= 0xFFFF )
         {
           put_octet ( 0 , 0x82 ) ;
           put_octet ( 1 , static_cast <Octet> ( length / 256 ) ) ;
           put_octet ( 2 , static_cast <Octet> ( length % 256 ) ) ;
         }
       else if ( length <= 0xFFFFFF )
         {
           put_octet ( 0 , 0x83 ) ;
           put_octet ( 1 , static_cast <Octet> ( (  length / 256 ) / 256) ) ;
           put_octet ( 2 , static_cast <Octet> ( (  length / 256 ) % 256 ) ) ;
           put_octet ( 3 , static_cast <Octet> ( length % 256 ) ) ;
         }
       else // TODO: handle lengths greater than than 0xFFFFFFFF?
         {
           put_octet ( 0 , 0x84 ) ;
           put_octet ( 1 , static_cast <Octet> ( (  ( length / 256 ) / 256) / 256) ) ;
           put_octet ( 2 , static_cast <Octet> ( (  ( length / 256 ) / 256) % 256) ) ;
           put_octet ( 3 , static_cast <Octet> ( (  length / 256 ) % 256 ) ) ;
           put_octet ( 4 , static_cast <Octet> ( length % 256 ) ) ;
         }
     };

  static BER_TLV_Length 
  generate ( Octet_String & where , const BER_TLV_Length & length )
    {
      Octet_String the_generated_length ( where , where . stop ( ) - ( length . the_stop - length . the_start ) ) ;
      Octet_String::cpy ( the_generated_length , length ) ;
      where = Octet_String ( where , where . start ( ) , the_generated_length . start ( ) ) ;
      return the_generated_length ;
    } ;

  static BER_TLV_Length 
  generate ( Octet_String & where , size_t length )
    // TODO: This should test that the octet is a valid single-octet length
    // TODO: If not, it should make up a multi-byte length
    {
      return generate ( where , BER_TLV_Length ( length ) ) ;
    } ;

   bool operator == ( const BER_TLV_Length & other) const
   {
      return ( Octet_String) ( *this) == ( Octet_String)other;
   } ;

   size_t value ( void ) const
   {
      switch ( the_stop - the_start )
      {
         case 1:
            return get_octet ( 0 ) ;
         case 2:
            return get_octet ( 1 ) ;
         case 3:
         default:
            return get_u16 ( 1 ) ;
      }
   }
} ;


class TLV_DLL Primitive_BER_TLV ;

class TLV_DLL Constructed_BER_TLV ;

class TLV_DLL BER_TLV;

class TLV_DLL Wrong_BER_TLV_Pointer_Type_Exception { } ;

class TLV_DLL BER_TLV_Pointer : public indexed_pointer_base
  {
  public:
    // This would be an enum were space not at a premium
    typedef Octet Type ;

    static const Type generic      = 0 ;

    static const Type primitive    = 1 ;

    static const Type constructed  = 2 ;

    BER_TLV_Pointer ( )
      : the_type ( generic )
      { } ;

    BER_TLV_Pointer ( const BER_TLV_Pointer & _Other )
      {
        attach ( Memory::index_to_pointer ( _Other . the_object_table_index ) ) ;
        the_type = _Other . the_type ;
      } ;

    BER_TLV_Pointer ( Primitive_BER_TLV * primitive_BER_TLV_pointer )
      {
        attach ( primitive_BER_TLV_pointer ) ;
        the_type = primitive ;
      } ;

    BER_TLV_Pointer ( Constructed_BER_TLV * constructed_BER_TLV_pointer )
      {
        attach ( constructed_BER_TLV_pointer ) ;
        the_type = constructed ;
      } ;

    ~BER_TLV_Pointer ( ) 
      { detach ( object_pointer ( ) ) ; } ;

    static BER_TLV_Pointer null ( ) { return BER_TLV_Pointer ( ) ; } ;

    BER_TLV_Pointer & operator = ( const BER_TLV_Pointer & _Other ) 
      {
        detach ( Memory::index_to_pointer (          the_object_table_index ) ) ;
        attach ( Memory::index_to_pointer ( _Other . the_object_table_index ) ) ;
        return * this ;
      } ;

    const BER_TLV * operator -> ( ) const 
      throw ( Wrong_BER_TLV_Pointer_Type_Exception ) ;

    BER_TLV * operator -> ( )
      throw ( Wrong_BER_TLV_Pointer_Type_Exception ) ;

    Type get_type ( ) const { return the_type ; } ;

  protected:
    void attach ( void * the_object_pointer )
      { add_reference_to_referenced_object ( the_object_pointer  ) ; } ;

    void detach ( void * the_object_pointer ) ;
   
    Type the_type ;
  } ;

class TLV_DLL BER_TLV : public TLV
{
public:

   // Create a Constructed_BER_TLV or a Primitive_BER_TLV
   // depending on the outermost tag
   static BER_TLV_Pointer create ( const Octet_String & octet_string , Memory::offset_t start );
   static BER_TLV_Pointer create ( const Octet_String & octet_string , Memory::offset_t start , Memory::offset_t stop);

   const BER_TLV_Tag tag ( void ) const { return the_tag ; } ;
   const BER_TLV_Length length ( void ) const { return the_length ; } ;
   size_t total_length ( void ) const { return the_tag.length ( )+the_length.length ( )+the_length.value ( ) ; } ;

   string spell ( const string & values_spelling ) const
   {
      string s ;
      s += "{ Tag=" ;
      s += the_tag . spell ( ) ;
      s += " Length=" ;
      s += the_length . spell ( ) ;
      s += " Values=" ;
      s += values_spelling ;
      s += " }" ;
      return s;
  }

protected:
  // The constructors are protected so that only subclasses can be instantiated
   BER_TLV ( const Octet_String & octet_string )
      : TLV ( octet_string , octet_string . start ( ) ),
        the_tag ( BER_TLV_Tag ( octet_string ) ) ,
        the_length ( BER_TLV_Length ( octet_string , the_tag . stop ( ) ) )
   { };


   BER_TLV ( const Octet_String & octet_string , Memory::offset_t start )
      : TLV ( octet_string , start ),
        the_tag ( BER_TLV_Tag ( octet_string , start ) ) ,
        the_length ( BER_TLV_Length ( octet_string , the_tag . stop ( ) ) )
   { };

   BER_TLV_Tag the_tag; 
   BER_TLV_Length the_length; 
} ;


class TLV_DLL Primitive_BER_TLV_Pointer : public indexed_pointer < Primitive_BER_TLV > 
  {
  public:
    Primitive_BER_TLV_Pointer ( ) { } ;
    Primitive_BER_TLV_Pointer ( const BER_TLV_Pointer & BER_TLV_pointer )
      {
        if ( BER_TLV_Pointer::primitive == BER_TLV_pointer . get_type ( ) ) 
          attach ( BER_TLV_pointer . object_pointer ( ) );
        else
          throw Wrong_BER_TLV_Pointer_Type_Exception ( ) ;
      } ;
  } ;

class TLV_DLL Primitive_BER_TLV : public BER_TLV
{
public:
  friend class Primitive_BER_TLV_Pointer ;

  // TODO: These don't check the parsed tag for being a Primitive_BER_TLV_Tag!
  Primitive_BER_TLV ( const Octet_String & octet_string )
  throw ( Octet_String_Index_Exception , Malformed_TLV_Exception )
    : BER_TLV ( octet_string , octet_string . start ( ) ) ,
      the_values ( Octet_String ( octet_string , the_length . stop ( ) , 
                                  the_length . stop ( ) + the_length . value ( ) ) )
    {
      the_stop = the_values . stop ( ) ;
    }

  Primitive_BER_TLV ( const Octet_String & octet_string , Memory::offset_t start )
  throw ( Octet_String_Index_Exception , Malformed_TLV_Exception )
    : BER_TLV ( octet_string , start ) ,
      the_values ( Octet_String ( octet_string , the_length . stop ( ) , 
                                  the_length . stop ( ) + the_length . value ( ) ) )
    {
      the_stop = the_values . stop ( ) ;
    }

  Primitive_BER_TLV ( const Octet_String &octet_string , Memory::offset_t start , Memory::offset_t stop) 
    throw ( Octet_String_Index_Exception , Malformed_TLV_Exception )
    : BER_TLV ( octet_string , start ) ,
      the_values ( Octet_String ( octet_string , the_length . stop ( ) , 
                                  the_length . stop ( ) + the_length . value ( ) ) )
    {
      the_stop = the_values . stop ( ) ;
      if ( the_stop < stop )
      {
          throw Malformed_TLV_Exception ( "too many bytes", octet_string, start, stop, the_stop );
      }
      else if ( stop < the_stop )
      {
          throw Malformed_TLV_Exception ( "not enough bytes", octet_string, start, stop, the_stop );
      }
    }

  Primitive_BER_TLV ( const Octet single_octet_tag , Octet * octet_string  , size_t length )
  throw ( Octet_String_Index_Exception , Malformed_TLV_Exception )
    : BER_TLV ( single_octet_tag ) ,
      the_values ( Octet_String ( octet_string , length ) )
    {
      the_stop = the_values . stop ( ) ;
    }
    
  static Primitive_BER_TLV
  generate ( Octet_String & where , const BER_TLV_Tag & tag , const BER_TLV_Length & length )
    {
      Octet_String the_generated_tlv ( where , 
                                       where . stop ( ) 
                                         -  ( (  tag . stop ( )  - tag . start ( ) ) 
                                              + ( length . stop ( )  - length . start ( ) ) 
                                              + length . value ( ) ) );
      where = Octet_String ( where , 
                             where . start ( ) , 
                             where . stop ( ) - length . value ( ) ) ;
      BER_TLV_Length::generate ( where , length ) ;
      BER_TLV_Tag::generate ( where , tag ) ;
      return the_generated_tlv ;
    }
 
  static Primitive_BER_TLV
  generate ( Octet_String & where , const BER_TLV_Tag & tag , 
             const BER_TLV_Length & length , Octet_String & values )
    {
      Octet_String::cpy(where, where . stop ( ) - length . value ( ),  
                        values , 0 , length . value ( ) );
      return generate ( where , tag , length ) ;
    }

  static Primitive_BER_TLV
  generate ( Octet_String & where , const BER_TLV_Tag & tag , const size_t length )
    {
      return generate ( where , tag , BER_TLV_Length ( length ) ) ;
    }
   
  static Primitive_BER_TLV
  generate ( Octet_String & where , const Octet single_octet_tag , const BER_TLV_Length & length )
    {
      return generate ( where , BER_TLV_Tag ( single_octet_tag ) , length ) ;
    }
   
  static Primitive_BER_TLV
  generate ( Octet_String & where , const Octet single_octet_tag , const size_t length )
    {
      return generate ( where , BER_TLV_Tag ( single_octet_tag ) , BER_TLV_Length ( length ) ) ;
    }
   
   
  string spell ( ) const { return BER_TLV::spell ( spell_values ( ) ) ; } ;

  string spell_values ( ) const
   {
      if ( the_values . start ( ) == the_values . stop ( ) )
      {
         return "<none>";
      }
      else
      {
         return the_values . spell ( ) ;
      }
   } ;
   
   Octet_String & values ( ) 
   {
      return the_values ;
   }

   const Octet_String & values ( ) const
   {
      return the_values ;
   }

   
protected:
   Octet_String the_values;
} ;



class TLV_DLL BER_TLV_Sequence : public Octet_String
{
private:
  typedef list < BER_TLV_Pointer , 
                 persistent_allocator < BER_TLV_Pointer> >  
    BER_TLV_Pointer_list ;
//  typedef list< BER_TLV_Pointer >  BER_TLV_Pointer_list ;

public:
   typedef BER_TLV_Pointer_list::const_iterator iterator;

   // This might need to be more complicated if other padding bytes are allowed
   static const int padding = 0x00;

   static bool is_padding ( Octet o ) { return o == padding ; } ;

   BER_TLV_Sequence ( const Octet_String & octet_string, Memory::offset_t start , Memory::offset_t stop)
      : Octet_String ( octet_string, start , stop )
   {
      while ( start < stop  )
      {
         if ( is_padding ( octet_string [ start - octet_string . start ( ) ] ) ) // skip padding
         {
            start ++ ;
         }
         else
         {
            BER_TLV_Pointer the_next_BER_TLV = BER_TLV::create ( octet_string , start )  ;
            the_TLVs . push_back ( the_next_BER_TLV ) ;
            start = the_next_BER_TLV -> stop ( ) ;
         }
      }
   }

   iterator begin ( void) const
   {
      return the_TLVs . begin ( );
   }

   iterator end ( void) const
   {
      return the_TLVs . end ( );
   }

   string spell ( ) const ;

    BER_TLV_Pointer find ( const BER_TLV_Tag & tag ) const
    {
      BER_TLV_Sequence::iterator p;

      for ( p = begin ( ) ; p != end ( ) ; p ++ )
        if ( ( * p ) -> tag ( ) == tag )
          return * p;

      return BER_TLV_Pointer::null ( );
    } ;

private:
   BER_TLV_Pointer_list the_TLVs;
} ;


class TLV_DLL Constructed_BER_TLV_Pointer : public indexed_pointer < Constructed_BER_TLV > 
  {
  public:
    Constructed_BER_TLV_Pointer ( const BER_TLV_Pointer & BER_TLV_pointer )
      {
        if ( BER_TLV_Pointer::constructed == BER_TLV_pointer . get_type ( ) ) 
          attach ( BER_TLV_pointer . object_pointer ( ) );
        else
          throw Wrong_BER_TLV_Pointer_Type_Exception ( ) ;
      } ;
  } ;


class TLV_DLL Constructed_BER_TLV : public BER_TLV
  {
  public:
    friend class Constructed_BER_TLV_Pointer ;

    Constructed_BER_TLV ( const Octet_String & octet_string  )
    throw ( Octet_String_Index_Exception , Malformed_TLV_Exception )
      : BER_TLV ( octet_string , octet_string . start ( ) ) ,
        the_values ( BER_TLV_Sequence ( octet_string , the_length . stop ( ) , 
                                        the_length . stop ( ) + the_length . value ( ) ) )
    {
      the_stop = the_values . stop ( ) ;
    }

    Constructed_BER_TLV ( const Octet_String & octet_string , Memory::offset_t start )
    throw ( Octet_String_Index_Exception , Malformed_TLV_Exception )
      : BER_TLV ( octet_string , start ) ,
        the_values ( BER_TLV_Sequence ( octet_string , the_length . stop ( ) , 
                                        the_length . stop ( ) + the_length . value ( ) ) )
    {
      the_stop = the_values . stop ( ) ;
    }

    Constructed_BER_TLV ( const Octet_String & octet_string , Memory::offset_t start , Memory::offset_t stop) 
    throw ( Octet_String_Index_Exception , Malformed_TLV_Exception )
      : BER_TLV ( octet_string , start ) ,
        the_values ( BER_TLV_Sequence ( octet_string , the_length . stop ( ) , 
                                        the_length . stop ( ) + the_length . value ( ) ) )
    {
      the_stop = the_values . stop ( ) ;
      if ( the_stop < stop )
      {
          throw Malformed_TLV_Exception ( "too many bytes", octet_string, start, stop, the_stop );
      }
      else if ( stop < the_stop )
      {
          throw Malformed_TLV_Exception ( "not enough bytes", octet_string, start, stop, the_stop );
      }
    }

    static Constructed_BER_TLV 
    generate ( Octet_String & where , const Constructed_BER_TLV & ber_tlv )
      {
        Octet_String the_generated_tlv ( where , where . stop ( ) - ( ber_tlv . the_stop - ber_tlv . the_start ) );
        Octet_String::cpy ( the_generated_tlv , ber_tlv ) ;
        where = Octet_String ( where , where . start ( ) , the_generated_tlv . start ( ) ) ;
        return the_generated_tlv ;
      }

  static Constructed_BER_TLV
  generate ( Octet_String & where , const BER_TLV_Tag & tag , const BER_TLV_Length & length )
    {
      Octet_String the_generated_tlv ( where , 
                                       where . stop ( ) 
                                         -  (   ( tag . stop ( ) - tag . start ( ) ) 
                                              + ( length . stop ( ) - length . start ( ) ) 
                                              + length . value ( ) ) );
      where = Octet_String ( where , 
                             where . start ( ) , 
                             where . stop ( ) - length . value ( ) ) ;
      BER_TLV_Length::generate ( where , length ) ;
      BER_TLV_Tag::generate ( where , tag ) ;
      return the_generated_tlv ;
    }

    string spell ( ) const { return BER_TLV::spell ( spell_values ( ) ) ; } ;

    string spell_values ( ) const
    {
        if ( the_values . start ( ) == the_values . stop ( ) )
        {
          return "<none>";
        }
        else
        {
          return the_values . spell ( ) ;
        }
    } ;

    const BER_TLV_Sequence & values ( ) const
    {
        return the_values ;
    } ;

    BER_TLV_Pointer find ( const BER_TLV_Tag & tag ) const
      {  return the_values . find ( tag ) ; } ;

    BER_TLV_Pointer find ( const Octet single_octet_tag ) const
    {
      return find ( BER_TLV_Tag ( single_octet_tag ) ) ;
    } ;


  protected:
    BER_TLV_Sequence the_values;
  } ;


#if SIMPLE_TLV_NOT_USED
/* ISO/IEC 7816-4 1-byte SIMPLE tag */
class TLV_DLL SIMPLE_TLV_Tag : public Octet_String
{
public:
   SIMPLE_TLV_Tag ( const Octet_String & octet_string, Memory::offset_t start )
      :  Octet_String ( octet_string , start , start + 1 )
   { } ;
   bool operator == ( const SIMPLE_TLV_Tag & other) const
   {
      return ( Octet_String) ( *this)== ( Octet_String)other;
   }
} ;



class TLV_DLL SIMPLE_TLV_Length : public Octet_String
{
public:
   SIMPLE_TLV_Length ( const Octet_String & octet_string, Memory::offset_t start)
      : Octet_String ( octet_string , start , 
                      0xFF != octet_string [ start ] 
                         ? start + 1  
                         : start + 3 )
   { } ;

   bool operator == ( const SIMPLE_TLV_Length & other) const
   {
      return ( Octet_String) ( *this)== ( Octet_String)other;
   } ;

   size_t value ( void ) const
   {
      if ( the_start + 1 == the_stop )
      {
         return get_octet ( 0 ) ;
      }
      else
      {
         return get_u16 ( 1 ) ;
      }
   }
} ;


class TLV_DLL SIMPLE_TLV : public TLV
{
public:
   SIMPLE_TLV ( const Octet_String & octet_string , Memory::offset_t start )
	   throw ( Octet_String_Index_Exception , Malformed_TLV_Exception )
      : TLV ( octet_string , start ) ,
        the_tag ( SIMPLE_TLV_Tag ( octet_string , start ) ) ,
        the_length ( SIMPLE_TLV_Length ( octet_string , the_tag . stop ( ) ) ) ,
        the_values ( Octet_String ( octet_string , the_length . stop ( ) , 
                                    the_length . stop ( ) + the_length . value ( ) ) ) 
   {
      the_stop = the_values . stop ( )  ;
   } ;

   SIMPLE_TLV ( const Octet_String & octet_string , Memory::offset_t start , Memory::offset_t stop) 
	   throw ( Octet_String_Index_Exception , Malformed_TLV_Exception )
      : TLV ( octet_string , start ) ,
        the_tag ( SIMPLE_TLV_Tag ( octet_string , start ) ) ,
        the_length ( SIMPLE_TLV_Length ( octet_string , the_tag . stop ( ) ) ) ,
        the_values ( Octet_String ( octet_string , the_length . stop ( ) , 
                                    the_length . stop ( ) + the_length . value ( ) ) ) 
   {
      the_stop = the_values . stop ( )  ;
      if ( the_stop < stop )
      {
         throw Malformed_TLV_Exception ( "too many bytes", octet_string, start, stop, the_stop );
      }
      else if ( stop < the_stop )
      {
         throw Malformed_TLV_Exception ( "not enough bytes", octet_string, start, stop, the_stop );
      }
   } ;

   string spell ( void ) const
   {
      string s ;
      s = "{ Tag=" ;
      s += the_tag . spell ( ) ;
      s += " Length=" ;
      s += the_length . spell ( ) ;
      s += " Values=" ;
      if ( the_values . start ( ) == the_values . stop ( ) )
      {
         s += "<none>";
      }
      else
      {
         s += the_values . spell ( ) ;
      }
      s += " }" ;
      return s ;
   }

   const SIMPLE_TLV_Tag tag ( void ) const { return the_tag ; } ;

   const SIMPLE_TLV_Length length ( void ) const { return the_length ; } ;

protected:
   const SIMPLE_TLV_Tag the_tag; 
   const SIMPLE_TLV_Length the_length; 
   const Octet_String the_values;
} ;

class TLV_DLL SIMPLE_TLV_Sequence : public Octet_String
{
private:
   typedef list<SIMPLE_TLV , persistent_allocator<SIMPLE_TLV> > SIMPLE_TLV_list;
public:
   typedef SIMPLE_TLV_list::const_iterator iterator;

   SIMPLE_TLV_Sequence ( const Octet_String & octet_string, Memory::offset_t start , Memory::offset_t stop)
      : Octet_String ( octet_string, start , stop )
   {
      while ( start < stop  )
      {
         if ( 0x00 == octet_string [ start ] ) // skip padding
         {
            start ++ ;
         }
         else
         {
            SIMPLE_TLV & a_SIMPLE_TLV = * new SIMPLE_TLV ( octet_string , start ) ;
            the_TLVs . push_back ( a_SIMPLE_TLV ) ;
            start = a_SIMPLE_TLV . stop ( ) ;
         }
      }
   }

   iterator begin ( void) const
   {
      return the_TLVs . begin ( ) ;
   }

   iterator end ( void) const
   {
      return the_TLVs . end ( ) ;
   }

private:
   SIMPLE_TLV_list the_TLVs;
} ;

#endif  // SIMPLE_TLV_NOT_USED


#endif   // !defined(__TLV_H_INCLUDED__)
