//
// The APDU Client Interface
//
// Storage -- Persistent, reference-counted memory
//
// COPYRIGHT
// ---------
// The software was developed by the National Institute of Standards and 
// Technology (NIST), an agency of the Federal Government.  Pursuant to 
// Title 15 United States Code Section 105, works of NIST are not subject
// to copyright protection in the United States and are considered to be
// in the public domain.  As a result, a formal license is not needed to
// use the software.
// 
// The Government Smart Card Reference Implementation software is provided
// by NIST as a service and is expressly provided "AS IS."  
// NIST MAKES NO WARRANTY OF ANY KIND, EXPRESS, IMPLIED OR STATUTORY, 
// INCLUDING, WITHOUT LIMITATION, THE IMPLIED WARRANTY OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE, NON-INFRINGEMENT AND DATA ACCURACY.
// NIST does not warrant or make any representations regarding the use of
// the software or the results thereof, including but not limited to the
// correctness, accuracy, reliability or usefulness of the software.
// 
// Permission to use this software is contingent upon your acceptance of 
// the terms of this agreement and upon your providing appropriate
// acknowledgments of NIST's ownership of the software.

//
// Utility function to exchange an APDU, logging in a standard format
// The exchange is performed by * APDU_ClientPtr
// The cAPDU is sent from * Command
// The rAPDU is received into * Response
// The exchange is logged into logfile
// The client is requested to place comment in its log file
// The client exchange result is stored into *Result
// The function returns 1 if the exchange is completed 
//    and the status in *Reponse is 0x9000 or 0x91xx, 0 otherwise
//
#if !defined(__APDU_CLIENT_H_INCLUDED__)
#define __APDU_CLIENT_H_INCLUDED__
#include <APDU_Server.h>
#include <stdio.h>

class APDU_Client 
{
public:
  APDU_Client ( APDU_Server & server )
    : the_APDU_server ( server ) ,
      the_verbosity ( 0 ) /* Change to 0 for silence */
    { } ;

  virtual void set_verbosity ( int verbosity ) 
    {
      the_verbosity = verbosity ;
      if ( 0 < verbosity ) verbosity -- ;
      the_APDU_server . set_verbosity ( verbosity ) ;
    } ;

  virtual long initialize ( int verbosity ) 
    {
      set_verbosity ( verbosity ) ;
      if ( 0 < verbosity ) verbosity -- ;
      return the_APDU_server . initialize ( verbosity ) ;
    }

  virtual long run ( void ) = 0 ;

  virtual long finalize ( void )
    {
      return the_APDU_server . finalize ( ) ;
    };

  virtual long exchange_APDUs ( const CAPDU_Packet & command, 
                                RAPDU_Packet & response , 
                                const char * comment ) ;

protected:
  int the_verbosity ;
  APDU_Server & the_APDU_server ;
} ;

#endif  // !defined(__APDU_CLIENT_H_INCLUDED__)
