// APDU Server
//
// COPYRIGHT
// ---------
// The software was developed by the National Institute of Standards and 
// Technology (NIST), an agency of the Federal Government.  Pursuant to 
// Title 15 United States Code Section 105, works of NIST are not subject
// to copyright protection in the United States and are considered to be
// in the public domain.  As a result, a formal license is not needed to
// use the software.
// 
// The Government Smart Card Reference Implementation software is provided
// by NIST as a service and is expressly provided "AS IS."  
// NIST MAKES NO WARRANTY OF ANY KIND, EXPRESS, IMPLIED OR STATUTORY, 
// INCLUDING, WITHOUT LIMITATION, THE IMPLIED WARRANTY OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE, NON-INFRINGEMENT AND DATA ACCURACY.
// NIST does not warrant or make any representations regarding the use of
// the software or the results thereof, including but not limited to the
// correctness, accuracy, reliability or usefulness of the software.
// 
// Permission to use this software is contingent upon your acceptance of 
// the terms of this agreement and upon your providing appropriate
// acknowledgments of NIST's ownership of the software.

#if !defined(__APDU_SERVER_H_INCLUDED__)
#define __APDU_SERVER_H_INCLUDED__

#include <stddef.h>
#include <stdio.h>

#include <APDU_Packet.h>

typedef long APDU_Handler ( const CAPDU_Packet & command , 
                                  RAPDU_Packet & response ) ; 

class APDU_Server 
{
public:
  static const int MAX_ATR_LENGTH = 32 ;  // Per ISO 7816

  virtual long exchange_APDUs   ( const CAPDU_Packet & command , 
                                        RAPDU_Packet & response )    = 0 ;

  virtual long initialize       ( int verbosity )             = 0 ;
  virtual long finalize         ( void )                      = 0 ;
  virtual long open_log_file    ( const char * filename )     = 0 ;
  virtual long flush_log_file   ( void )                      = 0 ;
  virtual long close_log_file   ( void )                      = 0 ;
  virtual long log_file_comment ( const char * comment )      = 0 ;
  virtual void set_verbosity    ( int verbosity )             = 0 ;
  virtual int  get_verbosity    ( void )                      = 0 ;
  virtual long power_up         ( void )                      = 0 ;
  virtual long power_down       ( void )                      = 0 ;
  virtual void get_ATR          ( unsigned char * ATR , 
                                  unsigned int & ATR_length ) = 0 ;
  // Store whatever persistent state is needed as binary output to 
  // the_persistent_storage_output_file, updating the value of checksum
  virtual bool store_persistent_state ( FILE * the_persistent_storage_output_file , 
                                        unsigned short & checksum ) = 0 ;
  // Restore whatever persistent state is needed as binary input from
  // the_persistent_storage_input_file, updating the value of checksum
  virtual bool load_persistent_state ( FILE * the_persistent_storage_input_file , 
                                       unsigned short & checksum ) = 0 ;

} ;


#endif // !defined(__APDU_SERVER_H_INCLUDED__)
