// TLP224 Input and Output
//
// COPYRIGHT
// ---------
// The software was developed by the National Institute of Standards and 
// Technology (NIST), an agency of the Federal Government.  Pursuant to 
// Title 15 United States Code Section 105, works of NIST are not subject
// to copyright protection in the United States and are considered to be
// in the public domain.  As a result, a formal license is not needed to
// use the software.
// 
// The Government Smart Card Reference Implementation software is provided
// by NIST as a service and is expressly provided "AS IS."  
// NIST MAKES NO WARRANTY OF ANY KIND, EXPRESS, IMPLIED OR STATUTORY, 
// INCLUDING, WITHOUT LIMITATION, THE IMPLIED WARRANTY OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE, NON-INFRINGEMENT AND DATA ACCURACY.
// NIST does not warrant or make any representations regarding the use of
// the software or the results thereof, including but not limited to the
// correctness, accuracy, reliability or usefulness of the software.
// 
// Permission to use this software is contingent upon your acceptance of 
// the terms of this agreement and upon your providing appropriate
// acknowledgments of NIST's ownership of the software.

#if !defined(__TLP224_H_INCLUDED__)
#define __TLP224_H_INCLUDED__



#if defined(_WIN32)
#define WIN32_LEAN_AND_MEAN
#include <winsock2.h>
#else
#error Non-windows include(s)?
#endif

#define BUF_SIZE 4096

class TLP224
  {
  public:

    // TPL-224 Commands
    const static unsigned char TLPC_SET_MODE                  = 0x01 ;  // 00 [OB]
    const static unsigned char TLPC_CONFIGURE_SIO_LINE        = 0x0A ;  // CB
    const static unsigned char TLPC_SET_DELAY                 = 0x23 ;  // 01 00 4C 01 Delay
    const static unsigned char TLPC_READ_FIRMWARE_VERSION     = 0x22 ;  // 05 3F E0 10
    const static unsigned char TLPC_POWER_DOWN                = 0x4D ;  // ROS Format  Card 0 only
    const static unsigned char TLPC_POWER_DOWN_CARD_0         = 0x11 ;  // GCC Format  Card 0
    const static unsigned char TLPC_POWER_DOWN_CARD_1         = 0x19 ;  // GCC Format  Card 1
    const static unsigned char TLPC_POWER_UP                  = 0x6E ;  // 00 00 00   ROS Format  Card 0 only
    const static unsigned char TLPC_POWER_UP_CARD_0           = 0x12 ;  // [CFG] [PTS0,PTS1,PTS2,PTS3,PC]  GCC Format  Card 0
    const static unsigned char TLPC_POWER_UP_CARD_1           = 0x1A ;  // [CFG] [PTS0,PTS1,PTS2,PTS3,PC]  GCC Format  Card 1
    const static unsigned char TLPC_INPUT_FROM_MICROPROCESSOR = 0xDB ;  // CLA INS P1 P2 Le   ROS Format  Card 0 only
    const static unsigned char TLPC_ISO_OUTPUT_CARD_0         = 0x13 ;  // CLA INS P1 P2 Le   GCC Format  Card 0
    const static unsigned char TLPC_ISO_OUTPUT_CARD_1         = 0x1B ;  // CLA INS P1 P2 Le   GCC Format  Card 1
    const static unsigned char TLPC_OUTPUT_TO_MICROPROCESSOR  = 0xDA ;  // CLA INS P1 P2 Lc <data>  ROS Format  Card 0 only
    const static unsigned char TLPC_ISO_INPUT_CARD_0          = 0x14 ;  // CLA INS P1 P2 Lc <data>  GCC Format  Card 0
    const static unsigned char TLPC_ISO_INPUT_CARD_1          = 0x1C ;  // CLA INS P1 P2 Lc <data>  GCC Format  Card 1

    const static unsigned char TLPC_EXCHANGE_APDU_CARD_0      = 0x15 ;  // <Case 1, 2S, 3S or 4S APDU>  ROS & GCC  Card 0 
    const static unsigned char TLPC_EXCHANGE_APDU_CARD_1      = 0x1D ;  // <Case 1, 2S, 3S or 4S APDU>  ROS & GCC  Card 1

    const static unsigned char TLPC_DEFINE_CARD_TYPE          = 0x02 ;  // T  ROS Format Card 0 only (omit T for status?)
    const static unsigned char TLPC_DEFINE_CARD_0_TYPE        = 0x17 ;  // T  GCC Format Card 0      (omit T for status)
    const static unsigned char TLPC_DEFINE_CARD_1_TYPE        = 0x1F ;  // T  01 GCC Format Card 1   (omit T for status)

    const static unsigned char TLPC_TERMINATE                 = 0x56 ;  // or in diner decimal, 86

    const static unsigned char TLPC_SELECT_CARD_NUMBER        = 0x67 ;  // logical_card_number_high_byte logical_card_number_low_byte 
    const static unsigned char TLPC_LOAD_CARD_IMAGE           = 0x68 ;  // logical_card_number_high_byte logical_card_number_low_byte <card image file path>
    const static unsigned char TLPC_STORE_CARD_IMAGE          = 0x69 ;  // logical_card_number_high_byte logical_card_number_low_byte <card image file path>
    const static unsigned char TLPC_OPEN_LOG_FILE             = 0x6A ;  // <log file path>
    const static unsigned char TLPC_FLUSH_LOG_FILE            = 0x6B ;  // 
    const static unsigned char TLPC_CLOSE_LOG_FILE            = 0x6C ;  // 
    const static unsigned char TLPC_LOG_FILE_COMMENT          = 0x6D ;  // <ascii characters> -- end-of-line will be added


    //  TPL-224 Status Bytes
    const static unsigned char OSB_OK                        = 0x00 ;
    const static unsigned char OSB_CARD_NOT_SUPPORTED        = 0xA2 ;
    const static unsigned char OSB_CARD_INTERRUPTED_EXCHANGE = 0xE5 ;
    const static unsigned char OSB_OTHER_THAN_JUST_OK        = 0xE7 ;
    const static unsigned char OSB_CARD_NOT_PRESENT          = 0xFB ;

    TLP224 ( ) ;

    bool receive_TLP224_packet ( SOCKET & connection_socket , unsigned char & command , int & length ) ;

    bool send_TLP224_packet ( SOCKET & connection_socket , unsigned char OSB , int length ) ;

    bool get_debug_TLP224_traffic ( );

    bool set_debug_TLP224_traffic ( bool b );

    bool initialize_socket_io ( SOCKET & socket );

    bool initialize_socket_server ( SOCKET & socket_card_socket ) ;

    bool initialize_socket_client ( SOCKET & socket_card_socket, 
                                    const char * host, 
                                    u_short port , 
                                    bool report_error , 
                                    bool report_success ) ;

    void finalize_socket_io ( SOCKET & socket );

    bool put_byte_into_output_buffer ( unsigned char b );

    bool get_byte_from_input_buffer ( unsigned char & b );

    void reset_output_buffer ( );

    const char * status_code_description ( unsigned char OSB ) ;

  private:
    static bool debug_TLP224_traffic ;

    unsigned char buffer [ BUF_SIZE ] ;
    unsigned char * buffer_end ;
    unsigned char * buffer_pointer ;

    bool power_on ;

    unsigned char output_buffer [ BUF_SIZE ] ;
    unsigned char * output_buffer_end ;
    unsigned char * output_buffer_pointer ;

    unsigned char input_buffer [ BUF_SIZE ] ;
    unsigned char * input_buffer_end ;
    unsigned char * input_buffer_pointer ;

    const static unsigned char TLP_ACK = 0x60 ;

    void analyze_socket_io_error ( ) ;
    bool recv_buffer ( SOCKET & connection_socket ) ;
    bool send_buffer ( SOCKET & connection_socket ) ;
    bool get_socket_byte ( unsigned char & b ) ;
    void put_socket_byte ( unsigned char b ) ;

  };

#endif  // !defined(__TLP224_H_INCLUDED__)
