// TLP_APDU_Client -- TLP224 Listener
//
// COPYRIGHT
// ---------
// The software was developed by the National Institute of Standards and 
// Technology (NIST), an agency of the Federal Government.  Pursuant to 
// Title 15 United States Code Section 105, works of NIST are not subject
// to copyright protection in the United States and are considered to be
// in the public domain.  As a result, a formal license is not needed to
// use the software.
// 
// The Government Smart Card Reference Implementation software is provided
// by NIST as a service and is expressly provided "AS IS."  
// NIST MAKES NO WARRANTY OF ANY KIND, EXPRESS, IMPLIED OR STATUTORY, 
// INCLUDING, WITHOUT LIMITATION, THE IMPLIED WARRANTY OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE, NON-INFRINGEMENT AND DATA ACCURACY.
// NIST does not warrant or make any representations regarding the use of
// the software or the results thereof, including but not limited to the
// correctness, accuracy, reliability or usefulness of the software.
// 
// Permission to use this software is contingent upon your acceptance of 
// the terms of this agreement and upon your providing appropriate
// acknowledgments of NIST's ownership of the software.

#if !defined(__TLP_APDU_SERVER_H_INCLUDED__)
#define __TLP_APDU_SERVER_H_INCLUDED__

#include <APDU_Server.h>
#include <TLP224.h>

class TLP_APDU_Server: public APDU_Server
  {
  public:
    static const int MAX_ATR_LENGTH = 32 ;
    static const int DEFAULT_APDU_PORT = 9025 ;
    static const int MAX_HOSTNAME_BUFFER_LENGTH = 64;  
    static const char default_hostname [ ];

    virtual long initialize         ( int v ) ;
    virtual APDU_Handler exchange_APDUs ;
    virtual long finalize           ( void ) ;
    virtual long open_log_file      ( const char * filename ) ;
    virtual long flush_log_file     ( void );
    virtual long close_log_file     ( void );
    virtual long log_file_comment   ( const char * comment );
    virtual void set_verbosity      ( int v );
    virtual int  get_verbosity      ( void );
    virtual long power_up           ( void );
    virtual long power_down         ( void );
    virtual void get_ATR            ( unsigned char * ATR , 
                                      unsigned int & ATR_length ) ;
    virtual bool store_persistent_state ( FILE * the_persistent_storage_output_file , 
                                          unsigned short & checksum ) ;
    virtual bool load_persistent_state ( FILE * the_persistent_storage_input_file , 
                                        unsigned short & checksum ) ;


    // Needed to bring up TLP address
    int  set_hostname               ( const char * hostname );
    const char * get_hostname       ( void );
    void set_port                   ( int p) { port = p ; } ;
    int get_port                    ( void ) { return port ; } ;

    bool report_initialization_errors ;

  private:
    int verbosity;
    const char * hostname;
     
    unsigned char the_ATR [ 32 ]; 
    int the_ATR_length ;
    int port;
    SOCKET card_edge_socket;
    CAPDU_Packet the_command ;
    RAPDU_Packet the_response;
    TLP224 the_TLP224 ;
  };

#endif  //  __TLP_APDU_SERVER_H_INCLUDED__
