// Hexify
//
// COPYRIGHT
// ---------
// The software was developed by the National Institute of Standards and 
// Technology (NIST), an agency of the Federal Government.  Pursuant to 
// Title 15 United States Code Section 105, works of NIST are not subject
// to copyright protection in the United States and are considered to be
// in the public domain.  As a result, a formal license is not needed to
// use the software.
// 
// The Government Smart Card Reference Implementation software is provided
// by NIST as a service and is expressly provided "AS IS."  
// NIST MAKES NO WARRANTY OF ANY KIND, EXPRESS, IMPLIED OR STATUTORY, 
// INCLUDING, WITHOUT LIMITATION, THE IMPLIED WARRANTY OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE, NON-INFRINGEMENT AND DATA ACCURACY.
// NIST does not warrant or make any representations regarding the use of
// the software or the results thereof, including but not limited to the
// correctness, accuracy, reliability or usefulness of the software.
// 
// Permission to use this software is contingent upon your acceptance of 
// the terms of this agreement and upon your providing appropriate
// acknowledgments of NIST's ownership of the software.

#if !defined(__HEXIFY_H_INCLUDED__)
#define __HEXIFY_H_INCLUDED__

#if defined(_WIN32) && defined(__cplusplus)
extern "C" {
#endif  /* defined(_WIN32) && defined(__cplusplus) */

#include <windows.h>
#include <ctype.h>
#include <string.h>

BOOL ishex ( char c);
BOOL isallhex ( const char *p ) ;
/* return the hex value of a char, or -1 if not a hex digit */
int hexvalue ( char c ) ;
/* return the hex char representing i, or '?' if not in 0..15 */
char hexchar ( unsigned int i ) ;
/*
 * The two functions below convert from hex to long, and return 0 if successful.
 * If a non-hex char is encountered at offset n in buf, they return n+1 as an error flag.
 */
int ahntol (const char * buf, const int nchars, unsigned long * pvalue);
int ahtol (const char * buf, unsigned long * pvalue);

/*
 * Convert to hex: long (either all 4 bytes or n bytes) and byte array
 */

char * ltoh ( const unsigned long value , char * buf ) ;
char * lntoh ( const unsigned long value , int n , char * buf ) ;
char * bntoh ( unsigned char * b , int n , char * buf ) ;

/*
 * add one string of bytes to another
 * the destination string should be at least as long as the addend string
 *
 * if the addend is shorter, the two string are added "right-aligned"
 * and the carry is propagated as necessary down the destination
 *
 * if the addend is longer, the last "dest_len" bytes are silently used.
 *
 * return the carry ( 0 or 1 )
 */
unsigned char add_binary ( unsigned char * dest , unsigned long int dest_len , unsigned char * addend , unsigned long int addend_len ) ;


#if defined(_WIN32) && defined(__cplusplus)
} /* extern "C" */
#endif  /* defined(_WIN32) && defined(__cplusplus) */

#endif /* __HEXIFY_H_INCLUDED__ */
