// PIV VSCR Service.cpp : Defines the entry point for the console application.
//

#include "stdafx.h"
#include "Winsvc.h"
#include "Debug.h"
#include "serviceThread.h"
#include "EventThread.h"
#include "PIV VSCR Service.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#endif

#define PIVVSCR_SERVICE_NAME			_T("PIV Virtual Smart Card Reader")

// The one and only application object

CWinApp theApp;

//using namespace std;

SERVICE_STATUS g_ServiceStatus;
SERVICE_STATUS_HANDLE g_ServiceStatusHandle = 0;

CServiceThread* g_pThread = NULL;
CEventThread* g_pEventThread = NULL;

void error_message(const char * format, ...)
{
	static char Buffer[4096 + 1] = {0};
	va_list vals;
	va_start ( vals , format ) ;     /* Initialize variable arguments. */
	_vsnprintf ( Buffer , 4096 , format , vals ) ;
	::ReportDefaultErrorEvent(g_AppName, Buffer);
}

void PIVVSCRWinServiceStop()
{
	// stop event thread
	if (g_pEventThread) {

		g_pEventThread->Stop();
		delete g_pEventThread;
		g_pEventThread = NULL;
	}

	// stop service thread
	if (g_pThread) {

		g_pThread->Stop();
		delete g_pThread;
		g_pThread = NULL;
	}

	ReportDefaultSuccessEvent(g_AppName, "PIV VSCR Service stopped.");
}

DWORD WINAPI PIVVSCRWinServiceCtrlHandler(DWORD dwControl, DWORD dwEventType, LPVOID lpEventData, LPVOID lpContext)
{
	DWORD dwStatus = NO_ERROR;

	switch (dwControl) {
	case SERVICE_CONTROL_SHUTDOWN:
	case SERVICE_CONTROL_STOP:
		// perform any processing necessary to stop the service
		PIVVSCRWinServiceStop();
		
		g_ServiceStatus.dwWin32ExitCode = 0;
		g_ServiceStatus.dwCurrentState = SERVICE_STOPPED;
		g_ServiceStatus.dwCheckPoint = 0;
		g_ServiceStatus.dwWaitHint = 0;

		// report service is stopped
		if (!SetServiceStatus(g_ServiceStatusHandle, &g_ServiceStatus)) {

			dwStatus = GetLastError();
			SvcDebugOut(_T("SetServiceStatus error in PIVVSCRWinServiceCtrlHandler.  Error code = "), 
				dwStatus);
		} 

		SvcDebugOut(_T("Stopped PIV Virtual Smart Card Reader Service"));
		return dwStatus;

	case SERVICE_CONTROL_INTERROGATE:
		// Fall through to send current status. 
		break; 

	default:
		SvcDebugOut(_T("Unrecognized control code "), dwControl);
		dwStatus = ERROR_CALL_NOT_IMPLEMENTED;
	} 

	// set current status
	if (!SetServiceStatus(g_ServiceStatusHandle, &g_ServiceStatus)) {

		SvcDebugOut(_T("SetServiceStatus error in PIVVSCRWinServiceCtrlHandler.  Error code = "), 
			GetLastError());
	}

	return dwStatus;
}

DWORD PIVVSCRWinServiceInitialization(DWORD argc, LPTSTR *argv, DWORD* specificError)
{
	DWORD dwStatus = ERROR_FUNCTION_FAILED;

	// TODO: perform initialization here
	::AddDefaultEventSource(g_AppName);
	::CreateRegistryKey(HKEY_LOCAL_MACHINE, PIVVSCR_PARAMETERS_REGKEY);

	// start our threads
	if ((g_pThread == NULL) && (g_pEventThread == NULL)) {
		DWORD dwPollInterval;

		// get poll interval
		if (::GetRegistryValue(
				HKEY_LOCAL_MACHINE, 
				PIVVSCR_PARAMETERS_REGKEY, 
				PIVVSCR_POLLINTERVAL_REGVALUE, 
				&dwPollInterval) == ERROR_SUCCESS) {

			// start event thread
			g_pEventThread = new CEventThread;
			dwStatus = g_pEventThread->Start();

			if (dwStatus == ERROR_SUCCESS) {

				// start service thread
				g_pThread = new CServiceThread(dwPollInterval);	// in milliseconds
				dwStatus = g_pThread->Start(argc, argv, specificError);

				if (dwStatus != ERROR_SUCCESS) {

					delete g_pThread;
					g_pThread = NULL;

					g_pEventThread->Stop();
					delete g_pEventThread;
					g_pEventThread = NULL;

					error_message("Failed to start service thread.\n");
				}
				else
					ReportDefaultSuccessEvent(g_AppName, "PIV VSCR Service started successfully.");
			}
			else {

				delete g_pEventThread;
				g_pEventThread = NULL;

				error_message("Failed to start event thread.\n");
			}
		}
		else {

			// log error in event log
			error_message("Could not get value of registry key %s at path \\\\HKEY_LOCAL_MACHINE\\%s\n",
				PIVVSCR_POLLINTERVAL_REGVALUE,
				PIVVSCR_PARAMETERS_REGKEY);
		}
	}

	return dwStatus;
}

void WINAPI PIVVSCRWinServiceStart(DWORD argc, LPTSTR *argv)
{
	DWORD dwStatus, dwSpecificError;

	// initialize service status structure
	g_ServiceStatus.dwServiceType = SERVICE_WIN32_SHARE_PROCESS;
	g_ServiceStatus.dwCurrentState = SERVICE_START_PENDING;
	g_ServiceStatus.dwControlsAccepted = SERVICE_ACCEPT_SHUTDOWN | SERVICE_ACCEPT_STOP;
	g_ServiceStatus.dwWin32ExitCode = 0;
	g_ServiceStatus.dwServiceSpecificExitCode = 0;
	g_ServiceStatus.dwCheckPoint = 0;
	g_ServiceStatus.dwWaitHint = 0;

	// register service control handler
	g_ServiceStatusHandle = RegisterServiceCtrlHandlerEx(
									PIVVSCR_SERVICE_NAME, 
									PIVVSCRWinServiceCtrlHandler, 
									NULL);

	if (g_ServiceStatusHandle == (SERVICE_STATUS_HANDLE)0) {

		SvcDebugOut(_T("RegisterServiceCtrlHandlerEx failed "), GetLastError());
		return;
	}

	// perform service initialization
	dwStatus = PIVVSCRWinServiceInitialization(argc, argv, &dwSpecificError); 

	if (dwStatus != ERROR_SUCCESS) {
		
		g_ServiceStatus.dwCurrentState = SERVICE_STOPPED;
		g_ServiceStatus.dwCheckPoint = 0;
		g_ServiceStatus.dwWaitHint = 0;
		g_ServiceStatus.dwWin32ExitCode = dwStatus;
		g_ServiceStatus.dwServiceSpecificExitCode = dwSpecificError;

		SetServiceStatus(g_ServiceStatusHandle, &g_ServiceStatus);
		return;
	}

	// initialization complete - report running status
	g_ServiceStatus.dwCurrentState = SERVICE_RUNNING;
	g_ServiceStatus.dwCheckPoint = 0;
	g_ServiceStatus.dwWaitHint = 0;

	if (!SetServiceStatus(g_ServiceStatusHandle, &g_ServiceStatus)) {

		SvcDebugOut(_T("SetServiceStatus error in PIVVSCRWinServiceStart.  Error code = "), 
			GetLastError());
	}
	else {

		// check if card should be automatically inserted during startup
		g_pThread->CheckAutoInsert();

		SvcDebugOut(_T("Service running"));
	}
}

int _tmain(int argc, TCHAR* argv[], TCHAR* envp[])
{
	int nRetCode = 0;

	// initialize MFC and print and error on failure
	if (!AfxWinInit(::GetModuleHandle(NULL), NULL, ::GetCommandLine(), 0))
	{
		// TODO: change error code to suit your needs
		_tprintf(_T("Fatal Error: MFC initialization failed\n"));
		nRetCode = 1;
	}
	else
	{
		SERVICE_TABLE_ENTRY DispatchTable[] = { {PIVVSCR_SERVICE_NAME, PIVVSCRWinServiceStart }, { NULL, NULL } };
 
		if (!StartServiceCtrlDispatcher(DispatchTable)) {
			
			SvcDebugOut(_T("StartServiceCtrlDispatcher error = "), 
				GetLastError());
		}
	}

	return nRetCode;
}
