// Virtual Smart Card Reader
//
// COPYRIGHT
// ---------
// The software was developed by the National Institute of Standards and 
// Technology (NIST), an agency of the Federal Government.  Pursuant to 
// Title 15 United States Code Section 105, works of NIST are not subject
// to copyright protection in the United States and are considered to be
// in the public domain.  As a result, a formal license is not needed to
// use the software.
// 
// The Government Smart Card Reference Implementation software is provided
// by NIST as a service and is expressly provided "AS IS."  
// NIST MAKES NO WARRANTY OF ANY KIND, EXPRESS, IMPLIED OR STATUTORY, 
// INCLUDING, WITHOUT LIMITATION, THE IMPLIED WARRANTY OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE, NON-INFRINGEMENT AND DATA ACCURACY.
// NIST does not warrant or make any representations regarding the use of
// the software or the results thereof, including but not limited to the
// correctness, accuracy, reliability or usefulness of the software.
// 
// Permission to use this software is contingent upon your acceptance of 
// the terms of this agreement and upon your providing appropriate
// acknowledgments of NIST's ownership of the software.

#if !defined(__TLP_SERVER_H_INCLUDED__)
#define __TLP_SERVER_H_INCLUDED__

#include "TLP224.h"
#include <fstream>
#include "EventLog.h"

class TLP_Server
{
public:
	static const int  MAX_ATR_LENGTH             = 32;
    static const int  DEFAULT_APDU_PORT          = 9025;
    static const int  MAX_HOSTNAME_BUFFER_LENGTH = 64;  
    static const char default_hostname [];

    long initialize( );
    long finalize(void);
    long power_up(void);
    long power_down(void);
    void get_ATR(BYTE *ATR, unsigned int &ATR_length);
	bool exchangeData(BYTE *, int, BYTE *, int &);

    int        set_hostname(const char *m_hostname);
	const char *get_hostname(void) { return m_hostname; };
    void       set_port(int p)     { m_port = p; };
    int        get_port(void)      { return m_port; };

    bool m_bIntError ;

  private:
    const char    *m_hostname;
    unsigned char m_ATR[32]; 
    int           m_ATRlength;
    int           m_port;
    SOCKET        m_scktCardEdge;
    TLP224        m_ServerTLP224IO;

    void error_message(const char * format, ...)
    {
        static char Buffer[4096 + 1] = {0};
        va_list vals;
        va_start ( vals , format ) ;     /* Initialize variable arguments. */
        _vsnprintf ( Buffer , 4096 , format , vals ) ;
        //MessageBox ( NULL , Buffer , "Virtual Smart Card Reader Interface" , MB_OK ) ;
		::ReportDefaultErrorEvent(g_AppName, Buffer);
    }
};

#endif  // !defined(__TLP_SERVER_H_INCLUDED__)